<?php

class Comparator
{
  private $dbClient;

  public function __construct(Client $dbClient)
  {
    $this->dbClient = $dbClient;
  }


  /**
   * @param array[ClientClass] $parsedClients
   * @param int $houseId
   * @return array
   */
  public function compareParsedClientsWithDatabase(array $parsedClients, int $houseId)
  {
    $resultArray = array();

    //DEBUG: print all parsed clients
    /*
    echo("Parsed clients from XML: <pre>");
    print_r($parsedClients);
    echo("</pre>");
    */
    ////////// Process clients
    $dbClients = $this->dbClient->get_clients_by_house_id($houseId);

    //DEBUG: print all (corresponding) clients from database
    /*
    echo("Filtered clients from DB: <pre>");
    print_r($dbClients);
    echo("</pre>");
    */

    $temp_used_pidz = array();
    $dupClients = array();
    $noOfDupClients = 0;

    foreach ($parsedClients as $parsedClient) {

      // Check by pid if parsed client is already in DB
      $dbClient = $this->parsed_client_in_db($parsedClient, $dbClients);

      // Add all dup clients to an temp array for adding the flat weight
      if ($this->parsed_client_equals_db_client($parsedClient, $dbClient)) {
        $dupClients[$noOfDupClients++] = $parsedClient;
      }

      // If object exists and its id wasn't used before
      if ($dbClient !== null && !in_array($dbClient["pid"], $temp_used_pidz)) {
        array_push($temp_used_pidz, $dbClient["pid"]); //push new pid to array

        // Copy parsedClient
        $copiedClient = clone $parsedClient;

        if ($this->parsed_client_equals_db_client($copiedClient, $dbClient)) {
          // Nothing to do here
          continue;
        } else {
          $copiedClient->setUpdateType(ClientUpdate::UPDATE);
          $copiedClient->isClientTable = true;
          array_push($resultArray, $copiedClient);
          continue;
        }
      }
    }

    ////////// Process correspondences
    $dbCorrespondences = $this->dbClient->get_full_correspondences_by_house_id($houseId);

    foreach ($parsedClients as $parsedClient) {
      $dbMatchingCorrespondences = $this->parsed_full_correspondences_in_db($parsedClient, $dbCorrespondences);

      // Parsed correspondences is not in database => create
      if ($dbMatchingCorrespondences == null) {
        $parsedClient->setUpdateType(ClientUpdate::CREATE);
        array_push($resultArray, $parsedClient);
        continue;
      }

      // check if data updated
      if ($dbMatchingCorrespondences !== null) {
        // Client with multiple correspondences (hack: check if array)
        if (!array_key_exists('first_name', $dbMatchingCorrespondences)) {
          $duplicate = false;
          foreach ($dbMatchingCorrespondences as $dbMatchingCorrespondence) {
            if ($this->parsed_corres_equals_db_corres($parsedClient, $dbMatchingCorrespondence)) {
              $duplicate = true;
            }
          }
          if ($duplicate === false) {
            $parsedClient->setUpdateType(ClientUpdate::CREATE);
            array_push($resultArray, $parsedClient);
            continue;
          }
          // Only one corres
        } else {
          if ($this->parsed_corres_equals_db_corres($parsedClient, $dbMatchingCorrespondences)) {
            continue;
          } else {
            $parsedClient->setUpdateType(ClientUpdate::CREATE);
            array_push($resultArray, $parsedClient);
            continue;
          }
        }
      }
    }

    // Check if clients in database can be deleted
    foreach ($dbCorrespondences as $dbCorrespondence) {
      if (!$this->is_db_corres_in_parsed_corres_list($dbCorrespondence, $parsedClients)) {

        // Delete pid for current hid-object-combination in correspondences table
        $toBeDeleted = new ClientClass();
        $toBeDeleted->setUpdateType(ClientUpdate::DELETE);
        $toBeDeleted->id = $dbCorrespondence['pid'];
        $toBeDeleted->firstName = $dbCorrespondence['first_name'];
        $toBeDeleted->surname = $dbCorrespondence['last_name'];
        $toBeDeleted->firstName2 = $dbCorrespondence['first_name2'];
        $toBeDeleted->surname2 = $dbCorrespondence['last_name2'];
        $toBeDeleted->company = $dbCorrespondence['company'];
        $toBeDeleted->mobil = $dbCorrespondence['mobil'];
        $toBeDeleted->email = $dbCorrespondence['email'];
        $toBeDeleted->email2 = $dbCorrespondence['email2'];
        $toBeDeleted->flat = $dbCorrespondence['flat'];
        $toBeDeleted->level = $dbCorrespondence['level'];
        $toBeDeleted->isCouncil = $dbCorrespondence['is_council'];
        $toBeDeleted->flatWeight = $dbCorrespondence['flat_weight'];
        $toBeDeleted->garageWeight = $dbCorrespondence['garage_weight'];

        array_push($resultArray, $toBeDeleted);
        continue;
      }
    }


    // Check if flat weight needs to be added
    foreach ($dupClients as $dupClient) {
      $dbMatchingCorrespondence =
        $this->is_parsed_client_correspondence_in_db($dupClient, $dbCorrespondences);

      // check if data updated
      if ($dbMatchingCorrespondence) {
        //if ( $dbMatchingCorrespondence['flat_weight'] == 0 || $dbMatchingCorrespondence['garage_weight'] == 0 ) {
        if ($dupClient->flatWeight != 0 || $dupClient->garageWeight != 0) {
          // Check if flatweight automatically should be updated
          // Second part is in parse_xml.php

          /*
          $dupClient->hasFlatWeightChanged = true;
          $dupClient->setUpdateType(ClientUpdate::UPDATE_FLAT_WEIGHT);
          array_push( $resultArray, $dupClient );
          */

        }
        //}
      }
    }

    return $resultArray;
  }


  private function parsed_client_in_db($parsedClient, $dbClients)
  {
    foreach ($dbClients as $dbClient) {
      if (strcasecmp($parsedClient->id, $dbClient['pid']) == 0) {
        return $dbClient;
      }
    }

    return null;
  }

  private function parsed_client_equals_db_client($parsedClient, $dbClient): bool
  {
    $equal = true;

    //TODO: in case of null, why are the other ifs not else-ifs?
    //I mean, all of them access an array that could be null -> potential null pointer
    //either change all ifs to else-ifs or add !is_null($dbClient) to all of them or similar
    //see notice: Trying to access array offset on value of type null
    if ($dbClient == null) 
      $parsedClient->newEntry = true;

    //TODO: PHP Notice:  Trying to access array offset on value of type null in 191, 196, 202, 208, 214, 222, 229, 235, 241 - possible fix: "if (isset($dbClient['pid']) && strcmp..." but what should be the else-case in case it is null, is null acceptable?

    if (strcmp(htmlspecialchars(strip_tags($parsedClient->id)), $dbClient['pid']) !== 0) {
      //echo "( " . $parsedClient->id . " vs " . $dbClient['pid'] . " ) <br>";
      $equal = false;
    }

    if (strcmp(htmlspecialchars(strip_tags($parsedClient->firstName)), $dbClient['first_name']) !== 0) {
      //echo "( " . $parsedClient->firstName . " vs " . $dbClient['first_name'] . " ) <br>";
      $equal = false;
      $parsedClient->hasFirstNameChanged = true;
    }

    if (strcmp(htmlspecialchars(strip_tags($parsedClient->surname)), $dbClient['last_name']) !== 0) {
      //echo "( " . $parsedClient->surname . " vs " . $dbClient['last_name'] . " ) <br>";
      $equal = false;
      $parsedClient->hasSurnameChanged = true;
    }

    if (strcmp(htmlspecialchars(strip_tags($parsedClient->firstName2)), $dbClient['first_name2']) !== 0) {
      //echo "( " . $parsedClient->firstName2 . " vs " . $dbClient['first_name2'] . " ) <br>";
      $equal = false;
      $parsedClient->hasFirstName2Changed = true;
    }

    if (strcmp(htmlspecialchars(strip_tags($parsedClient->surname2)), $dbClient['last_name2']) !== 0) {
      //echo "( " . $parsedClient->surname2 . " vs " . $dbClient['last_name2'] . " ) <br>";
      $equal = false;
      $parsedClient->hasSurname2Changed = true;
    }

    // Remove new lines from company-string and replace with one empty space
    $parsedClient->company = preg_replace('/\s+/', ' ', trim($parsedClient->company));
    if (strcmp(htmlspecialchars(strip_tags($parsedClient->company)), $dbClient['company']) !== 0) {
      //echo  "( " . htmlspecialchars(strip_tags($parsedClient->company)) . " vs " . $dbClient['company'] . " ) <br>";
      //echo $this->toAsciiCode(htmlspecialchars(strip_tags($parsedClient->company))) . " -  " . $this->toAsciiCode($dbClient['company']) . " = " . strcmp( htmlspecialchars(strip_tags($parsedClient->company)), $dbClient['company'] ) . "<br>";
      $equal = false;
      $parsedClient->hasIsCompanyChanged = true;
    }

    if (strcmp(htmlspecialchars(strip_tags($parsedClient->mobil)), $dbClient['mobil']) !== 0) {
      //echo "( " . $parsedClient->mobil . " vs " . $dbClient['mobil'] . " ) <br>";
      $equal = false;
      $parsedClient->hasMobilChanged = true;
    }

    if (strcmp(htmlspecialchars(strip_tags($parsedClient->email)), $dbClient['email']) !== 0) {
      //echo "( " . $parsedClient->email . " vs " . $dbClient['email'] . " ) <br>";
      $equal = false;
      $parsedClient->hasEmailChanged = true;
    }

    if (strcmp(htmlspecialchars(strip_tags($parsedClient->email2)), $dbClient['email2']) !== 0) {
      //echo "( " . $parsedClient->email2 . " vs " . $dbClient['email2'] . " ) <br>";
      $equal = false;
      $parsedClient->hasEmail2Changed = true;
    }

    return $equal;
  }

  private function parsed_full_correspondences_in_db($parsedClient, $dbCorrespondences)
  {
    $result = array();
    $count = 0;

    foreach ($dbCorrespondences as $dbCorrespondence) {
      if (strcasecmp($parsedClient->id, $dbCorrespondence['pid']) == 0) {
        array_push($result, $dbCorrespondence);
        $count++;
      }
    }

    if ($count == 0) {
      return null;
    }
    if ($count == 1) {
      return reset($result);
    } else {
      return $result;
    }
  }

  /* check one db entry in entire xml file
     if pid is same and data different, potentially it can be deleted (not found yet)
     if afterwards the equal one can be found, return true
     if the identical one cannot be found return false (isFound-state)  */

  private function parsed_corres_equals_db_corres($parsedClient, $dbCorres): bool
  {
    $equal = true;

    if (strcmp(htmlspecialchars(strip_tags($parsedClient->flat)), $dbCorres['flat']) !== 0) {
      $equal = false;
      //$parsedClient->hasFlatChanged = true; //MEMO: commented out because object is manipulated via different loops with same objects
    }

    if ($parsedClient->level != $dbCorres['level']) {
      $equal = false;
      //$parsedClient->hasLevelChanged = true;
    }

    if (strcmp(htmlspecialchars(strip_tags($parsedClient->isCouncil)), $dbCorres['is_council']) !== 0) {
      $equal = false;
      //$parsedClient->hasIsCouncilChanged = true;
    }

    if ($parsedClient->flatWeight != $dbCorres['flat_weight']) {
      $equal = false;
    }

    if ($parsedClient->garageWeight != $dbCorres['garage_weight']) {
      $equal = false;
    }

    return $equal;
  }

  private function is_db_corres_in_parsed_corres_list($dbCorres, $parsedClients): bool
  {
    $isFound = false;
    foreach ($parsedClients as $parsedClient) {
      if (strcasecmp($parsedClient->id, $dbCorres['pid']) == 0) {
        //the identical is eventually found (same pid, same data)
        if ($this->parsed_corres_equals_db_corres($parsedClient, $dbCorres))
          return true;
      }
    }
    return $isFound;
  }

  private function is_parsed_client_correspondence_in_db($parsedClient, $dbCorrespondences)
  {
    foreach ($dbCorrespondences as $dbCorrespondence) {
      if (strcasecmp($parsedClient->id, $dbCorrespondence['pid']) == 0) {
        if ($parsedClient->flat == $dbCorrespondence['flat']) {
          if ($parsedClient->level == $dbCorrespondence['level']) {
            return $dbCorrespondence;
          }
        }
      }
    }
    return null;
  }
}