<?php

use Monolog\Logger;

require_once 'models/RootInfoModel.php';
require_once 'models/RealEstateModel.php';
require_once 'models/HouseModel.php';
require_once 'models/FlatType.php';
require_once 'models/Level.php';
require_once 'models/FlatModel.php';
require_once 'models/ClientModel.php';

class OpenImmoXmlParser
{

  private $xmlDocument = null;
  private $xmlRoot = null;
  private $rootInfos = null;
  private $realEstate = null;
  private $log;

  public function __construct(Logger $log)
  {
    $this->log = $log;
  }

  public function parse($xmlData): ?RealEstateModel
  {
    if ($xmlData == null)
      return null;

    try {
      $this->xmlDocument = new SimpleXMLElement($xmlData);
    } catch (Exception $e) {
      error_log( "ERROR: xml_parser " . $e->getMessage(), 0 );
    }

    $nameSpaces = $this->xmlDocument->getNameSpaces(true);
    $this->xmlRoot = $this->xmlDocument->children($nameSpaces["imo"]);    //$xml->children('imo', TRUE)
    $this->parseRootInfos();
    $this->parseRealEstate();

    return $this->realEstate;
  }

  private function parseRootInfos()
  {
    $this->rootInfos = new RootInfoModel();
    $element = $this->xmlRoot->attributes();
    foreach ($element as $key => $value) {

      switch ($key) {
        case "sendersoftware":
          $this->rootInfos->setSenderSoftware($value);
          break;
        case "version":
          $this->rootInfos->setVersion($value);
          break;
      }
    }

  }

  private function parseRealEstate()
  {
    $element = $this->xmlRoot[1];
    $immos = $element->immobilie;
    $house = null;
    foreach ($immos as $immo) {
      $kind = $immo->objektkategorie->objektart;
      if (isset($kind->sonstige)) {
        $this->realEstate = RealEstateModel::fromXml($this->log, $immo, $this->rootInfos);
        $this->log->debug("<p>" . $this->realEstate);

      } else if (isset($kind->haus)) {
        $house = HouseModel::fromXml($immo);
        if ($house != null) {
          $this->realEstate->addHouse($house);
        }
        $this->log->debug("&nbsp&nbsp&nbsp&nbsp" . $house);

      } else if (isset($kind->wohnung)) {
        $flat = new FlatModel($this->log);
        $flat->fromXml($immo);
        $house->addFlat($flat);
        if (strcmp($flat->getAddressHash(), $house->getAddressHash()) != 0) {
          $this->log->error("PLZ, Ort + Straße zwischen Haus und Wohnung sind unterschiedlich.");
        }
        $this->log->debug("&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp" . $flat);
        foreach ($flat->getClients() as $client) {
          $this->log->debug("&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp" . $client);
        }
      }
    }
  }
}