<?php

use Monolog\Handler\AbstractProcessingHandler;
use Monolog\Logger;

class XmlParserLogHandler extends AbstractProcessingHandler
{
  const DETAIL_LOG = '<br><button onclick="Collapsible(this)" class="collapsible">Details</button><div class="content">';

  private $detailLog;
  private $detailLogWarnings;
  private $detailLogErrors;

  private $errorCount = 0;

  public function __construct($level = Logger::DEBUG, bool $bubble = true)
  {
    parent::__construct($level, $bubble);
  }

  public function getErrorCount(): int {
    return $this->errorCount;
  }

  public function getFormattedDetailLog(): string
  {
    $log = XmlParserLogHandler::DETAIL_LOG . $this->detailLog . '<br>' . $this->detailLogErrors . '<br>' .
      $this->detailLogWarnings . '</div></br>';

    // Clear afterwards.
    $this->detailLog = "";
    $this->detailLogWarnings = "";
    $this->detailLogErrors = "";
    return $log;
  }

  public function getResultLog($number): string
  {
    $logLine = "";
    if ($this->errorCount == 0) {
      $logLine .= "<br><br><font color=green><b>OK:</font></b> Einlesen abgeschlossen.<br>";

    } else {

      $logLine .= "<br><font color=red><b>FEHLER:</font></b> Fehler beim Einlesen. Anzahl der Fehler: $this->errorCount<br>";
      $logLine .= $this->detailLogErrors;
    }

    return $logLine;
  }

  protected function write(array $record): void
  {
    $logLine = $record['message'];

    // Debug logs
    if ($record['level_name'] == 'DEBUG') {
      $logLine = $record['message'] . "<br>";
    }

    // Info logs
    if ($record['level_name'] == 'INFO') {
      $logLine = $record['message'] . "<br>";
    }

    // Handle warning logs
    if ($record['level_name'] == 'WARNING') {
      $logLine = "<font color=orange>WARNING:</font> - " . $record['message'] . "<br>";
    }

    // Handle error logs
    if ($record['level_name'] == 'ERROR') {
      $this->errorCount++;
      $logLine = "<font color=red>ERROR</font> - " . $record['message'] . "<br>";
    }

    // If a detail log, no need to update;
    if ($this->processDetailLog($record, $logLine)) {
      return;
    }

    // If logLine with params, echo and return
    $logLineWithParams = $this->processParams($record);
    if ($logLineWithParams) {
      echo($logLineWithParams);
      return;
    }

    // Echo the line
    echo($logLine);
  }

  private function processDetailLog($record, $logLine): bool
  {
    if (array_key_exists('detail', $record['context'])) {
      if ($record['level_name'] == 'WARNING') {
        $this->detailLogWarnings .= $logLine;

      } else if ($record['level_name'] == 'ERROR') {
        $this->detailLogErrors .= $logLine;

      } else {
        $params = $this->processParams($record);
        if ($params) {
          $this->detailLog .= $params;

        } else {
          $this->detailLog .= $logLine;
        }
      }

      return true;
    }

    return false;
  }

  private function processParams($record): ?string
  {
    $lines = null;
    if (array_key_exists('params', $record['context'])) {
      $lines = ('<br><b>' . $record['message'] . '</b><br>');
      foreach ($record['context']['params'] as $key => $value) {
        $lines .= '<b>' . $key . '</b>: ' . $value . '<br>';
      }
      $lines .= '<br>';
    }

    return $lines;
  }
}