<?php

use Monolog\Logger;

class ClientModel
{
  private $id = null;
  private $level = Level::UNKNOWN;

  private $firstName = null;
  private $surName = null;
  private $firstName2 = null;
  private $surName2 = null;

  private $email = null;
  private $email2 = null;
  private $mobil = null;
  private $company = null;

  private $isCouncil = null;

  private $log = null;
  private $isValid = false;
  private $addressHash = null;

  public static function fromXml(Logger $log, $xmlData): ClientModel
  {
    $client = new static();
    $client->log = $log;

    $client->id = (int)$xmlData->personennummer;

    $addressType = (string)$xmlData->attributes()->adressart;
    if ($addressType == "BESITZER" || $addressType == "MIETER") {
      $client->level = Level::RENTER;

      if ($addressType == "BESITZER") {
        $client->level = Level::OWNER;
      }
      // Set valid if addressType is 'Mieter' or 'Besitzer'
      // Ignore others like 'AltMieter'
      $client->isValid = true;
    }

    $client->company = (string)$xmlData->firma;
    $client->firstName = (string)$xmlData->vorname;
    $client->surName = (string)$xmlData->name;
    $client->firstName2 = (string)$xmlData->vorname2;
    $client->surName2 = (string)$xmlData->name2;
    $client->mobil = (string)$xmlData->tel_handy;
    $client->email = (string)$xmlData->email_direkt;
    $client->email2 = (string)$xmlData->email_zentrale;

    if ($xmlData->beruf == "Beirat" || $xmlData->beruf2 == "Beirat") {
      $client->isCouncil = 1;
    } else {
      $client->isCouncil = 0;
    }

    // Check if all important fields are set
    if (empty($client->id) || (empty($client->firstName) && empty($client->surname) && empty($client->company)) ||
      $client->id == 0 || $client->id == "" || ($client->firstName == "" && $client->surName == "" && $client->company == "")) {
      $log->warning("MissingData: <i>id=" . $client->id . " | Vorname=" . $client->firstName .
        " | Nachname= " . $client->surName . " | Firma=" . $client->company . "</i>", ['detail' => true]);
    }

    // Generate a hash for the address to validate with the corresponding house.
    $address = $xmlData->plz . $xmlData->ort . $xmlData->strasse;
    $client->addressHash = sha1($address);

    return $client;
  }

  public function isValid(): bool
  {
    return $this->isValid;
  }


  /**
   * @return null
   */
  public function getId()
  {
    return $this->id;
  }

  /**
   * @return int
   */
  public function getLevel(): int
  {
    return $this->level;
  }

  /**
   * @return string
   */
  public function getFirstName(): ?string
  {
    return $this->firstName;
  }

  /**
   * @return string
   */
  public function getSurName(): ?string
  {
    return $this->surName;
  }

  /**
   * @return string
   */
  public function getFirstName2(): ?string
  {
    return $this->firstName2;
  }

  /**
   * @return string
   */
  public function getSurName2(): ?string
  {
    return $this->surName2;
  }

  /**
   * @return string
   */
  public function getEmail(): ?string
  {
    return $this->email;
  }

  /**
   * @return string
   */
  public function getEmail2(): ?string
  {
    return $this->email2;
  }

  /**
   * @return string
   */
  public function getMobil(): ?string
  {
    return $this->mobil;
  }

  /**
   * @return string
   */
  public function getCompany(): ?string
  {
    return $this->company;
  }

  /**
   * @return string
   */
  public function getIsCouncil(): bool
  {
    return $this->isCouncil;
  }

  public function __toString()
  {
    return "ClientModel() id=" . $this->id . ", firstName=" . $this->firstName . ", surName=" .
      $this->surName . ", company " . $this->company . ", level=" . $this->level;
  }

}