<?php

use Monolog\Logger;

class FlatModel
{
  private $number = null;
  private $title = null;

  private $type = FlatType::UNKNOWN;

  private $flatWeight = 0;
  private $garageWeight = 0;

  private $normaWeight= 0;
  private $officeWeight = 0;
  private $businessWeight = 0;

  private $businessNumber = null;
  private $garageNumber = null;

  private $clients = array();

  private $log;
  private $addressHash = null;

  public function __construct(Logger $log)
  {
    $this->log = $log;
  }

  public function fromXml($xmlData)
  {
    // Assure the number is set and not empty. Otherwise, number is not null anymore.
    if (isset($xmlData->geo->wohnungsnr) && !empty($xmlData->geo->wohnungsnr)) {
      $this->number = $xmlData->geo->wohnungsnr;
    }
    $this->title = $xmlData->freitexte->objekttitel;
    $this->parseSimpleFields($xmlData->verwaltung_objekt);
    $this->addressHash = sha1($xmlData->geo->plz . $xmlData->geo->ort . $xmlData->geo->strasse);

    // Add all weights together
    $this->flatWeight = (int)$this->flatWeight + (int)$this->businessWeight 
      + (int)$this->normaWeight + (int)$this->officeWeight;

    // Overwrite the flat number if a business number is set
    if ($this->businessNumber != null && !empty($this->businessNumber)) {
      $this->number = $this->businessNumber;
    }

    foreach ($xmlData->weitere_adresse as $element) {
      $client = ClientModel::fromXml($this->log, $element);
      if ($client->isValid()) {
        array_push($this->clients, $client);
      }
    }
  }

  private function parseSimpleFields($xmlData)
  {
    foreach ($xmlData->user_defined_simplefield as $value) {

      $field = $value->attributes()->feldname;

      if ($field == "6 1000-stel Wohnung") {
        $this->flatWeight = (float)$value * (float)100;
        $this->type = FlatType::FLAT;
      }

      if ($field == "7 1000-stel Büro") {
        $this->officeWeight = (float)$value * (float)100;
      }

      if ($field == "8 1000-stel Norma") {
        $this->normaWeight = (float)$value * (float)100;
      }

      if ($field == "9 1000-stel Gewerbe") {
        $this->businessWeight = (float)$value * (float)100;
      }

      if ($field == "7 1000-stel Garage") {
        $this->garageWeight = (float)$value * (float)100;
      }

      if ($field == "7 1000-stel Tiefgarage") {
        $this->garageWeight = (float)$value * (float)100;
      }

      if ($field == "Gewerbe-Nummer") {
        $this->businessNumber = (string)$value;
      }

      if ($field == "Garagen-Nummer") {
        $this->garageNumber = (string)$value;
      }
    }
  }

  public function getAddressHash()
  {
    return $this->addressHash;
  }

  /**
   * @return ClientModel[]
   */
  public function getClients(): array
  {
    return $this->clients;
  }

  /**
   * @return string
   */
  public function getNumber(): ?string
  {
    return $this->number;
  }

  /**
   * @return null
   */
  public function getTitle()
  {
    return $this->title;
  }

  /**
   * @return int
   */
  public function getType(): int
  {
    return $this->type;
  }

  /**
   * @return int
   */
  public function getFlatWeight(): ?int
  {
    return $this->flatWeight;
  }

  /**
   * @return int
   */
  public function getGarageWeight(): ?int
  {
    return $this->garageWeight;
  }

  /**
   * @return int
   */
  public function getNormaWeight(): ?int
  {
    return $this->normaWeight;
  }

  /**
   * @return int
   */
  public function getOfficeWeight(): ?int
  {
    return $this->officeWeight;
  }

  /**
   * @return int
   */
  public function getBusinessWeight(): ?int
  {
    return $this->businessWeight;
  }

  /**
   * @return int
   */
  public function getBusinessNumber(): ?int
  {
    return $this->businessNumber;
  }

  /**
   * @return int
   */
  public function getGarageNumber(): ?int
  {
    return $this->garageNumber;
  }

  public function __toString()
  {
    return "FlatModel() number=" . $this->number . ", type=" . FlatType::toString($this->type) .
      ", title=" . $this->title . ", flatWeight=" . $this->flatWeight . ", garageWeight=" . $this->garageWeight .
      ", businessWeight=" . $this->businessWeight . ", officeWeight=" . $this->officeWeight;
  }


}