<?php

class HouseModel
{

  private $houseId;
  private $title;
  private $zip;
  private $city;
  private $street;
  private $number;
  private $flats = [];

  private $addressHash;

  public static function fromXml($xmlData): HouseModel
  {
    $house = new static();

    $house->title = (string)$xmlData->freitexte->objekttitel;
    $house->zip = (int)$xmlData->geo->plz;
    $house->city = (string)$xmlData->geo->ort;
    $house->street = (string)$xmlData->geo->strasse;
    $house->number = (string)$xmlData->geo->hausnummer;
    $house->houseId = (int)$xmlData->verwaltung_techn->objektnr_intern;

    $house->addressHash = sha1($xmlData->geo->plz . $xmlData->geo->ort . $xmlData->geo->strasse);

    return $house;
  }

  public function getAddressHash(): string
  {
    return $this->addressHash;
  }

  public function addFlat(FlatModel $flat)
  {
    array_push($this->flats, $flat);
  }

  /**
   * @return FlatModel[]
   */
  public function getFlats(): array
  {
    return $this->flats;
  }

  /**
   * @return string
   */
  public function getHouseId(): ?string
  {
    return $this->houseId;
  }

  /**
   * @return string
   */
  public function getTitle(): ?string
  {
    return $this->title;
  }

  /**
   * @return string
   */
  public function getStreet(): ?string
  {
    return $this->street;
  }

  /**
   * @return int
   */
  public function getZip(): ?int
  {
    return $this->zip;
  }

  /**
   * @return string
   */
  public function getNumber(): ?string
  {
    return $this->number;
  }

  /**
   * @return string
   */
  public function getCity(): ?string
  {
    return $this->city;
  }

  public function __toString()
  {
    return "HouseModel() objectId=" . $this->houseId . ", title=" . $this->title . ", zip=" . $this->zip . ", city=" . $this->city . ", street="
      . $this->street . ", number=" . $this->number;
  }

}
