<?php

use Monolog\Logger;

class RealEstateModel
{
  private $rootInfos = null;

  private $objectId = null;
  private $title = null;
  private $zip = null;
  private $city = null;
  private $owner = null;
  private $houses = [];

  private $log;

  public static function fromXml(Logger $log, $xmlData, RootInfoModel $rootInfos): RealEstateModel
  {
    $realEstate = new static();
    $realEstate->log = $log;

    $realEstate->rootInfos = $rootInfos;

    $realEstate->objectId = (int)$xmlData->verwaltung_techn->objektnr_extern;
    $realEstate->title = $xmlData->freitexte->objekttitel;
    $realEstate->zip = $xmlData->geo->plz;
    $realEstate->city = $xmlData->geo->ort;

    if ($xmlData->weitere_adresse) {
      $client = ClientModel::fromXml($realEstate->log, $xmlData->weitere_adresse);
      if ($client->isValid()) {
        $realEstate->owner = $client;
      }
    }

    return $realEstate;
  }

  public function addHouse(HouseModel $house)
  {
    array_push($this->houses, $house);
  }

  public function getOwner(): ?ClientModel
  {
    return $this->owner;
  }

  public function getRootInfos(): RootInfoModel
  {
    return $this->rootInfos;
  }

  /**
   * @return HouseModel[]
   */
  public function getHouses(): array
  {
    return $this->houses;
  }

  /**
   * @return int
   */
  public function getObjectId(): int
  {
    return $this->objectId;
  }

  /**
   * @return string
   */
  public function getTitle(): string
  {
    return $this->title;
  }

  /**
   * @return null
   */
  public function getZip()
  {
    return $this->zip;
  }

  /**
   * @return null
   */
  public function getCity()
  {
    return $this->city;
  }

  public function __toString()
  {
    return "RealEstateModel() objectId=" . $this->objectId . ", title=" . $this->title . ", zip=" . $this->zip .
      ", city=" . $this->city;
  }

}