<?php 
  // Headers
  header('Access-Control-Allow-Origin: *');
  header('Content-Type: application/json');
  header('Access-Control-Allow-Methods: POST');
  header('Access-Control-Allow-Headers: Access-Control-Allow-Headers,Content-Type,Access-Control-Allow-Methods, Authorization, X-Requested-With');

  include_once '../config/database.php';
  include_once '../models/client.php';
  include_once '../models/helper.php';

  // Instantiate DB & connect
  $database = new Database();
  $db = $database->connect();

  // Instantiate blog client object
  $client = new Client($db);

  // Get ID
  //$client->id = isset($_GET['id']) ? $_GET['id'] : die();

  // Get raw posted data
  $data = json_decode(file_get_contents("php://input"));

  $rawhash = urldecode( $data->hash );
  $rawtoken = urldecode( $data->token );

  //remove whitespaces
  $client->hash = str_replace(' ', '', $rawhash);
  $client->token = str_replace(' ', '', $rawtoken);

  /*
    test cases:
    - new token, no hash
    - new token, hash exists
    - token exists, no hash
    - token exsits, hash exists, active is true
    - token exsits, hash exists, active is false
    + if token already is registered in device (should never happen because of is_registered)
  */

  //check if token is already registred
  $stmtToken = $client->is_token_registered();
  if( Helper::arrayContainsItems( $stmtToken ) ) {
    error_log( "ERROR: 409 Conflict in register_login (hash-token not found), Hash: " . $data->hash , 0);
    //token already registred
    header("HTTP/1.1 409 Conflict");
    return;
  }

  $stm = $client->is_hash_registered();
  // check if hash is registered in client db
  $hash_exists = Helper::arrayContainsItems( $stm );

  //register
  if( $hash_exists )
    $client->register();

  // login
  $stmt = $client->login();
  $num = $stmt->rowCount();

  // Create client
  if( Helper::arrayContainsItems( $stmt ) && $hash_exists )
  {
    $result = $stmt->fetch(PDO::FETCH_ASSOC);
    $client->active = $result['active'];

    if( $client->active == 0 )
    {
      header("HTTP/1.1 403 Forbidden"); //active is false -> but registered
    }
    else
    {
      header("HTTP/1.1 200 OK");      
    }
  } 
  else 
  {
    header("HTTP/1.1 404 Not Found"); //hash not found in database (client) -> not registered
  }

