<?php 
  include_once '../../config/header_default.php';
  include_once '../../config/database.php';

  include_once '../../models/news.php';
  include_once '../../models/client.php';
  include_once '../../models/helper.php';

  // Instantiate DB & connect
  $database = new Database();
  $db = $database->connect();

  // Instantiate news object
  $news = new News($db);
  // Instantiate client object
  $client = new Client( $db );


  /**************************************************/

  // Get raw posted data and check if data exists
  $data = json_decode(file_get_contents("php://input"));
  if( !isset( $data) ) {
    error_log( "WARNING: GetNews body data were not found... deviceId: " . $hDeviceId, 0);
    header("HTTP/1.1 404 Not Found");
    exit();
  }

  /**************************************************/

  // get header data from header_default
  $client->hash = $hHash;
  $client->deviceId = $hDeviceId;

  /**************************************************/


  if( isset($data->noOfItems) )
    $news->noOfItems = urldecode( $data->noOfItems );
  else 
    $news->noOfItems = 20;

  if( isset($data->offset) )
    $news->offset = urldecode( $data->offset );
  else
    $news->offset = 0;

  $news->isCouncil = $client->get_council();

  if( isset($data->house) )
    $news->house_id = urldecode( $data->house ); //house id (filter)

  //if isCouncil less than 0 => probably hash is wrong (security check)
  //INFO: getAll-flag + value 1337 = security seed
  if( $news->isCouncil < 0 && (!isset($data->getAll) || $data->getAll != 1337) ) 
  {
    error_log( "WARNING: Security Check 1, deviceId" . $hDeviceId, 0);
    header("HTTP/1.1 404 Not Found");
    exit();
  }
  
  $result = null;

  //get all data - admin only
  if( isset($data->getAll) && $data->getAll == 1337 ) {
    if( Helper::isLocalhost() == true )
      $result = $news->get_all_news();
    else {
      error_log( "WARNING: Security Check 2, deviceId" . $hDeviceId, 0);
      header("HTTP/1.1 400 No Remote Access Allowed");
      exit(); //just for good measure      
    }
  }
  //get single data depending on house id(s)
  else {
    //get all data depending on house ids of client
    if( $news->house_id == 0 ) {
      //if there is no house id, get all available house ids via hash
      $news->house_id = $client->get_house_ids();
      $result = $news->get_news_by_ids();
    }
    //get news for requested house id
    else {
      //checks if house id is available for hash (security check) and gets level
      $client->house_id = $news->house_id;
      $house_level = $client->get_level_by_hash_hid();

      //error_log( "INFO: house-level debug: " . print_r( $house_level, true ), 0 );

      if($house_level !== false) { //if it exists or not (if not false, then an array)
        $news->level = $house_level; //house-level is an array here!
        $result = $news->get_news_by_id();      
      }
    }
  }


  // Check if any news
  if( Helper::arrayContainsItems( $result ) ) {
    // news array
    $news_arr = array();

    while($row = $result->fetch(PDO::FETCH_ASSOC)) {
      extract($row);

      if (isset($house_city)) { //check if property exists
        $news_item = array(
          'id' => (int)$nid,
          'title' => html_entity_decode($title),
          'date' => html_entity_decode($date),
          'text' => nl2br( html_entity_decode($text) ),
          'level' => (int)$level,
          'isCouncil' => (boolean)$isCouncil,
          'house_id' => (int)$hid,
          'house_city' => html_entity_decode($house_city)
        );

        // Push to "data"
        array_push($news_arr, $news_item);
      }
      else
      {
        if(isset($nid))
          error_log( "WARNING: GetNews issue... house_city is not set? - deviceId: " . $hDeviceId . " - results:" . $result->rowCount() . " - id: " . $nid . " - hash: " . $hHash, 0);
        else
          error_log( "WARNING: GetNews issue... house_city is not set? - deviceId: " . $hDeviceId . " - results:" . $result->rowCount() . " - hash: " . $hHash, 0);
      }
    }
    echo json_encode( $news_arr );
    exit();

  } else {
    
    //error_log( "WARNING: V2GetNews issue... Maybe no news available? - deviceId: " . $hDeviceId . " - results:" . $result->rowCount() . " - noOfItems:" . $news->noOfItems . " - hash: " . $hHash/* . " - offset:" . $news->offset . " - house_id:" . print_r($news->house_id, TRUE)*/, 0);
    echo json_encode( array() );
    exit();
  }