<?php 
  include_once '../../config/header_default.php';
  include_once '../../config/database.php';

  include_once '../../models/client.php';
  include_once '../../models/helper.php';

  // Instantiate DB & connect
  $database = new Database();
  $db = $database->connect();

  // Instantiate blog client object
  $client = new Client($db);


  /**************************************************/

  // get header data from header_default
  $client->hash = $hHash;
  $client->deviceId = $hDeviceId;

  /**************************************************/

  /*
    Login
    - 403 client not active
    - 406 hash does not exist
    - 401 deviceId not found
    - 406 hash is not the same as the one coming from db (via deviceId)
  */


  //check if hash exists and active
  $stmtActive = $client->is_hash_registered();
  if( Helper::arrayContainsItems( $stmtActive ) )
  {
    $resultActive = $stmtActive->fetch(PDO::FETCH_ASSOC);
    $client->active = $resultActive['active'];

    // check if customer is active or not
    if( $client->active == 0 ) {
      error_log( "WARNING: Customer is not active (via login), hash: " . $hHash, 0);
      header("HTTP/1.1 403 Forbidden"); //active is false
      exit();
    }
  } //hash does not exist
  else {
      error_log( "WARNING: Customer's hash does not exist, hash: " . $hHash, 0);
      header("HTTP/1.1 406 Not Acceptable");
      exit();
  }


  //check if device is registered
  $stmt = $client->is_device_registered();
  if( !Helper::arrayContainsItems( $stmt ) ) {
    error_log( "WARNING: While login, DeviceId not found: " . $hDeviceId, 0);
    header("HTTP/1.1 401 Unauthorized"); //device id did not exist
    exit();
  }
  else {
    //update last used time stamp of a device
    $client->update_last_used_device();

    //fetch data
    $result = $stmt->fetch(PDO::FETCH_ASSOC);

    //check if hash is correct
    if( !isset( $result['hash'] ) || $client->hash != $result['hash'] ) {

      if( !isset( $result['hash'] ) )
        error_log( "ERROR: While login, no Hash found for corresponding DeviceId: " . $hDeviceId, 0);
      else
        error_log( "ERROR: While login, Hash does not match to hash in db (according to deviceId): " . $hHash . " vs. " . $result['hash'] . " (db)", 0);

      header("HTTP/1.1 406 Not Acceptable"); //correct hash did not exist
      exit();
    }


    /**************************************************/ 
    // LOGGING IN
    
    include_once '../../models/ologging.php';
    // Instantiate log object
    $olog = new OLog($db);
    // Get raw posted data
    $olog->tag = "Login";
    $olog->message = "Login successful! hash=" . $hHash;
    $olog->deviceId = $client->deviceId;
    // Create olog
    $olog->create_log();
    /**************************************************/


    header("HTTP/1.1 200 OK");
    exit();
  }