<?php 
  include_once '../../config/header_default.php';
  include_once '../../config/database.php';

  include_once '../../models/poll.php';

  // Instantiate DB & connect
  $database = new Database();
  $db = $database->connect();

  // Instantiate poll object
  $poll = new Poll($db);


  /**************************************************/

  // Get raw posted data and check if data exists
  $data = json_decode(file_get_contents("php://input"));
  if( !isset( $data) ) {
    error_log( "WARNING: Register body data were not found... hash: " . $hHash, 0);
    header("HTTP/1.1 404 Not Found");
    exit();
  }

  /**************************************************/

  // get header data from header_default
  $poll->hash = $hHash;

  /**************************************************/


  if( isset( $data->phash ) )
    $poll->phash = $data->phash;

  $decision = "";
  $decision_id = 0;
  $answer = 0; // 0=not answered, otherwise y1,n2,a3
  $answered_decisions = 0;

  //based on the current active decision get data
  $decision_data = $poll->get_next_decision();

  if( isset($decision_data[1]) )
    $decision = $decision_data[1];
  if( isset($decision_data[0]) )
    $decision_id = $decision_data[0];
  if( isset($decision_data[2]) )
    $answer = $decision_data[2];
  if( isset($decision_data[3]) )
    $answered_decisions = $decision_data[3];

  if( !isset($decision) || $decision == null ) {
    error_log( "WARNING: poll_get_active_decision - decision not found! - phash: " . $poll->phash, 0);
    header("HTTP/1.1 404 Not Found");
    exit();
  }

  //poll pending - status 601
  if( $decision_id == 888 ) {
    //error_log( "INFO: poll_get_active_decision - Poll pending - phash: " . $poll->phash . " -  hash: " . $hHash, 0);
    header("HTTP/1.1 601 Poll pending");
    exit();
  }

  //poll complete - status 600
  if( $decision_id == 999 ) {
    //error_log( "INFO: poll_get_active_decision - Poll done (666) - phash: " . $poll->phash . " -  hash: " . $hHash, 0);
    header("HTTP/1.1 600 Poll complete");
    exit();
  }

  $no_of_qs = $poll->get_no_of_decisions();

  $answers = $poll->get_answers_by_decision_no( $decision_id );

  //get weight string (default, garage, flat)
  $weightStr = $poll->get_poll_setting_weight_by_phash_decision( $decision_id );

  $poll_arr = array();

  if( $answers != null ) {

    while( $row = $answers->fetch(PDO::FETCH_ASSOC) ) {
      extract( $row );

      //depending on weight setting, calculate weight
      $newWeight = 0;
      switch ($weightStr) {
          case "garage":
              $newWeight = (int)$garageWeight;
              break;
          case "flat":
              $newWeight = (int)$flatWeight;
              break;
          default: //"default"
              $newWeight = (int)$flatWeight + (int)$garageWeight;
      }

      //create item/row
      $poll_item = array(
        'answer_X' => html_entity_decode($answer_X),
        'weight' => (int)$newWeight
      );

      // Push answers set to array
      array_push( $poll_arr, $poll_item );
    }
  }

  /*
  / partly duplicated code to calc the weight-results in poll_get_results
  */
  $agreeWeight = 0;
  $declineWeight = 0;
  $abstainWeight = 0;
  $maxWeight = 0;
  foreach ($poll_arr as &$value) {
    switch( $value['answer_X'] ) {
      case "1": $agreeWeight += $value['weight']; break;
      case "2": $declineWeight += $value['weight']; break;
      case "3": $abstainWeight += $value['weight']; break;
      default:;
    }
    $maxWeight += $value['weight'];
  }

  if( $maxWeight > 0 ) {
    $result_agree = round( $agreeWeight * 100 / $maxWeight, 0 );
    $result_decline = round( $declineWeight * 100 / $maxWeight, 0 );
    $result_abstain = round( $abstainWeight * 100 / $maxWeight, 0 );
  }
  else {
    $result_agree = 0;
    $result_decline = 0;
    $result_abstain = 0;
  }

  $absentee_hash = "";
  $absentee_name = "";

  //check if user is eligible to vote for next decision
  $main_user_is_eligible_to_vote = $poll->is_user_eligible_to_vote( $decision_id, $weightStr );

  //check if user has one absentee or more
  $absenteeList = $poll->get_absentee_list_by_agenthash( $hHash );

  //if there are absentees check if we need to register them
  if( $absenteeList != null && count($absenteeList) > 0 ) {

    //check if those guys are already registered...
    for( $i=0; $i<count($absenteeList); $i++) {

      //WORKAROUND: abuse the hash of the current object for the absentee (mix of main user and absentee in this object)
      // better $poll->absentee_hash ?
      $poll->hash = $absenteeList[$i]['hash'];

      //check if user exists
      if( $poll->does_answer_entry_exist() == 0 ) {

        //error_log( "DEBUG: poll_get_active_decision - does_answer_entry_exist() - hash: " . $poll->hash . " - count(absenteeList): " . count($absenteeList), 0);

        //create/register user
        if( $poll->create_answer_entry() == 0 ) {
          error_log( "ERROR: poll_get_active_decision - Register absentee was not successful - phash: " . $poll->phash . " - hash: " . $poll->hash, 0);
        }
        else
          error_log( "INFO: poll_get_active_decision - Register absentee was successful, answer-entry for absentee (" . $poll->hash  . ") was created for phash: " . $poll->phash, 0);
      }
    }


    //check if user has answered (yes if answer>0) or is not allowed to answer, then absentees are next
    if( $answer > 0 || !$main_user_is_eligible_to_vote ) {

      //run once again through the array and check if someone needs to answer now
      for( $i=0; $i<count($absenteeList); $i++) {

        //WORKAROUND: abuse the hash... set one by one
        $poll->hash = $absenteeList[$i]['hash'];

        //based on the current active decision get data
        $decision_data = $poll->get_next_decision();

        //absentee answer for current decision
        $abs_answer = 0;
        if( isset($decision_data[2]) )
          $abs_answer = $decision_data[2];

        //get status to check if e.g. absentee is already done with all votes
        $decision_id = $decision_data[0];

        $absentee_is_eligible_to_vote = $poll->is_user_eligible_to_vote( $decision_id, $weightStr );

        //not yet answered
        if( $abs_answer == 0 && $decision_id != 999 && $absentee_is_eligible_to_vote ) {
          $absentee_hash = $absenteeList[$i]['hash']; //"0x13371337";
          $absentee_name = $absenteeList[$i]['name']; //"Tore Offermann";
          break;
        }
      }
    }

    //reset object hash
    $poll->hash = $hHash;
  }

  //check if main-user is read-only (because wrong object) or not
  //if absentee must send an answer, it is never read-only
  if( $absentee_hash != "" || $main_user_is_eligible_to_vote )
    $read_only = 0;
  else
    $read_only = 1;

  $jsonArray = array(
                      //TODO: check why html_entity_decode is needed "twice" here
                      'decision' => html_entity_decode(html_entity_decode($decision)),
                      'decision_id' => (int)$decision_id,
                      'answered_decisions' => html_entity_decode($answered_decisions),
                      'no_of_decisions' => (int)$no_of_qs,
                      'answer' => html_entity_decode($answer),
                      'result_agree' => (int)$result_agree,
                      'result_decline' => (int)$result_decline,
                      'result_abstain' => (int)$result_abstain,
                      'read_only' => (int)$read_only
                    );

  if( $absentee_hash != "" ) {
    $jsonArray["absentee_hash"] = html_entity_decode($absentee_hash);
    $jsonArray["absentee_name"] = html_entity_decode($absentee_name);
  }

  echo json_encode( $jsonArray );
  exit();