<?php 
  class Client {
    
    // DB stuff
    private $conn;
    private $tableClient = 'client';
    private $tableDevice = 'device';
    private $tableHouse = 'house';
    private $tableCorres = 'correspondence';
    private $tableObject = 'object';

    // Client Properties
    public $pid;
    public $hash;
    public $company;
    public $first_name;
    public $last_name;
    public $first_name2;
    public $last_name2;
    public $flat;
    public $house_id;
    public $house_name;
    public $level;
    public $isCouncil;
    public $flatWeight;
    public $garageWeight;
    public $active;
    public $created_at;
    public $object_id;
    
    public $token;
    public $deviceId;
    public $deviceInfo;

    public $email;
    public $email2;
    public $qr_mail_sent;

    // Constructor with DB
    public function __construct( $db ) {
      $this->conn = $db;
    }


    // Get all objects and clients
    public function get_correspondences() {
      // Create query
      $query = 'SELECT co.cid, h.city as house_city, o.street as house_street, o.number as house_number, cl.hash, cl.pid, cl.first_name, cl.last_name, cl.first_name2, cl.last_name2, cl.company, cl.email, cl.email2, cl.qr_mail_sent, co.flat, co.hid as house_id, co.level, co.isCouncil, co.flat_weight, co.garage_weight, cl.active, co.created_at, (Select Count(token) From ' . $this->tableDevice . ' d Where cl.hash = d.hash) as token_count
                FROM ' . $this->tableClient . ' cl, ' . $this->tableCorres . ' co
                LEFT JOIN ' . $this->tableHouse . ' h ON co.hid = h.hid
                LEFT JOIN ' . $this->tableObject . ' o ON co.oid = o.oid
                WHERE co.pid = cl.pid
                ORDER BY co.created_at DESC';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      $stmt->execute();

      return $stmt;
    }


    // Get email-1 (first) by client
    public function get_email_by_client() {
      // Create query
      $query = 'SELECT cl.email
                FROM ' . $this->tableClient . ' cl
                WHERE cl.hash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind ID
      $stmt->bindParam( 1, $this->hash );

      // Execute query
      if( $stmt->execute() ) {

          $row = $stmt->fetch(PDO::FETCH_ASSOC);
          return $row['email'];
      }

      return "";
    }


    // Get email-2 (second) by client
    public function get_email2_by_client() {
      // Create query
      $query = 'SELECT cl.email2
                FROM ' . $this->tableClient . ' cl
                WHERE cl.hash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind ID
      $stmt->bindParam( 1, $this->hash );

      // Execute query
      if( $stmt->execute() ) {

          $row = $stmt->fetch(PDO::FETCH_ASSOC);
          return $row['email2'];
      }

      return "";
    }


    // Get all objects and clients by a specific Hash (client)
    public function get_client() {
          // Create query
          $query = 'SELECT h.name as house_city, cl.first_name, cl.last_name, cl.first_name2, cl.last_name2, cl.company, co.flat, co.hid as house_id, co.level, co.isCouncil
                    FROM ' . $this->tableClient . ' cl, ' . $this->tableCorres . ' co
                    LEFT JOIN ' . $this->tableHouse . ' h ON co.hid = h.hid
                    LEFT JOIN ' . $this->tableObject . ' o ON co.oid = o.oid
                    WHERE cl.hash = ? AND co.pid = cl.pid';

          // Prepare statement
          $stmt = $this->conn->prepare($query);

          // Clean data
          $this->hash = htmlspecialchars(strip_tags($this->hash));

          // Bind ID
          $stmt->bindParam( 1, $this->hash );
          //$stmt->bindValue(1, $this->hash, PDO::PARAM_STR);

          // Execute query 
          $stmt->execute();

          return $stmt;
    }


    // Get client and house
    public function get_client_by_hash_houseId() {
      // Create query
      $query = 'SELECT h.city as house_city, h.name as house_name, cl.first_name, cl.last_name, cl.company, co.hid as house_id
                FROM ' . $this->tableClient . ' cl, ' . $this->tableCorres . ' co
                LEFT JOIN ' . $this->tableHouse . ' h ON co.hid = h.hid
                WHERE cl.hash = :hash AND co.hid = :house_id AND co.pid = cl.pid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));
      $this->house_id = htmlspecialchars(strip_tags($this->house_id));

      // Bind data
      $stmt->bindParam(':hash', $this->hash);
      $stmt->bindParam(':house_id', $this->house_id);

      // Execute query
      $stmt->execute();

      $clientsArray = array();
      while($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
        extract($row);
        $clientRow = array( "first_name" => $first_name, "last_name" => $last_name, "company" => $company, "house_city" => $house_city, "house_name" => $house_name );
        array_push( $clientsArray, $clientRow );
      }

      return $clientsArray;
    }


    public function get_clients_by_house_id( $house_id ) {
      // Create query
      $query = 'SELECT cl.pid, cl.first_name, cl.last_name, cl.first_name2, cl.last_name2, cl.company, cl.mobil, cl.email, cl.email2, co.isCouncil
                FROM ' . $this->tableClient . ' cl, ' . $this->tableCorres . ' co
                WHERE co.hid = ' . $house_id . ' AND cl.pid = co.pid
                GROUP BY cl.pid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      $stmt->execute();

      $clientsArray = array();
      while($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
        extract($row);
        $clientRow = array( "pid" => $pid, "first_name" => $first_name, "last_name" => $last_name, "first_name2" => $first_name2, "last_name2" => $last_name2, "company" => $company, "mobil" => $mobil, "email" => $email, "email2" => $email2, "isCouncil" => $isCouncil );
        array_push( $clientsArray, $clientRow );
      }

      return $clientsArray;
    }


    public function get_full_correspondences_by_house_id( $house_id ) {
      // Create query
      $query = 'SELECT DISTINCT cl.pid, cl.first_name, cl.last_name, cl.first_name2, cl.last_name2, cl.company, cl.mobil, cl.email, cl.email2, co.flat, co.level, co.isCouncil as is_council, co.flat_weight, co.garage_weight
                FROM ' . $this->tableClient . ' cl, ' . $this->tableCorres . ' co
                WHERE co.hid = ' . $house_id . ' AND cl.pid = co.pid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      $stmt->execute();

      $clientsArray = array();
      while($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
        extract($row);
        $clientRow = array( "pid" => $pid, "first_name" => $first_name, "last_name" => $last_name, "first_name2" => $first_name2, "last_name2" => $last_name2, "company" => $company, "mobil" => $mobil, "email" => $email, "email2" => $email2, "flat" => $flat, "level" => $level, "is_council" => $is_council, "flat_weight" => $flat_weight, "garage_weight" => $garage_weight );
        array_push( $clientsArray, $clientRow );
      }

      return $clientsArray;
    }


    // Get all clients
    public function get_clientz_with_email( $code ) {

      if( $code != 1337 )
        return;

      // Create query
      $query = 'SELECT cl.pid, cl.hash, cl.first_name, cl.last_name, cl.first_name2, cl.last_name2, cl.company, cl.email, cl.email2, cl.active, cl.qr_mail_sent
                FROM ' . $this->tableClient . ' cl
                WHERE cl.email != ""';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      $stmt->execute();

      return $stmt;
    }

    // Delete Client
    public function delete_correspondences() {
      
      // Create query
      $query = 'DELETE FROM ' . $this->tableCorres . ' 
                WHERE pid = :pid AND hid = :hid AND flat = :flat AND isCouncil = :isCouncil AND flat_weight = :flatWeight AND garage_weight = :garageWeight';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->pid = htmlspecialchars(strip_tags( $this->pid ));
      $this->house_id = htmlspecialchars(strip_tags( $this->house_id ));
      $this->flat = htmlspecialchars(strip_tags( $this->flat ));
      $this->isCouncil = htmlspecialchars(strip_tags( $this->isCouncil ));
      $this->flatWeight = htmlspecialchars(strip_tags( $this->flatWeight ));
      $this->garageWeight = htmlspecialchars(strip_tags( $this->garageWeight ));

      // Bind data
      $stmt->bindParam( ':pid', $this->pid );
      $stmt->bindParam( ':hid', $this->house_id );
      $stmt->bindParam( ':flat', $this->flat );
      $stmt->bindParam( ':isCouncil', $this->isCouncil );
      $stmt->bindParam( ':flatWeight', $this->flatWeight );
      $stmt->bindParam( ':garageWeight', $this->garageWeight );

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      error_log( "ERROR: Could not delete correspondence from database (delete_correspondences): " . $stmt->error , 0);

      return false;
    }

    // Create Client
    public function create_client() {

      /////////////////////////////////////////
      // Prepare statement for Client
      /////////////////////////////////////////
      //if name is already set -> update, otherwise insert
      $query = 'INSERT INTO ' . $this->tableClient . ' (pid, first_name, last_name, first_name2, last_name2, company, hash, mobil, email, email2)
                VALUES (:pid, :first_name, :last_name, :first_name2, :last_name2, :company, :hash, :mobil, :email, :email2)
                ON DUPLICATE KEY UPDATE pid = :pid'; //this means: do nothing

                // ON DUPLICATE KEY UPDATE pid = :pid1, first_name = :first_name1, last_name = :last_name1, company = :company1, hash = :hash1, mobil = :mobil1, email = :email1, email2 = :email3'

      /*$query1 = 'INSERT INTO ' . $this->tableCorres . ' (pid, hid, flat, level)
                VALUES (:pid2, :hid, :flat, :level) 
                ON DUPLICATE KEY UPDATE pid = :pid3, hid = :hid1, flat = :flat1, level = :level1';*/
                
      try {
        // begin the transaction
        $this->conn->beginTransaction();

        // Prepare statement for Client
        $stmt = $this->conn->prepare( $query );

        // Clean data
        $this->pid = htmlspecialchars(strip_tags( $this->pid ));
        $this->first_name = htmlspecialchars(strip_tags( $this->first_name ));
        $this->last_name = htmlspecialchars(strip_tags( $this->last_name ));
        $this->first_name2 = htmlspecialchars(strip_tags( $this->first_name2 ));
        $this->last_name2 = htmlspecialchars(strip_tags( $this->last_name2 ));
        $this->company = htmlspecialchars(strip_tags( $this->company ));
        $this->hash = $this->make_sha256( $this->first_name, $this->last_name, $this->company, $this->pid );
        $this->mobil = htmlspecialchars(strip_tags( $this->mobil ));
        $this->email = htmlspecialchars(strip_tags( $this->email ));
        $this->email2 = htmlspecialchars(strip_tags( $this->email2 ));

        // Bind data
        $stmt->bindParam( ':pid', $this->pid );
        $stmt->bindParam( ':first_name', $this->first_name );
        $stmt->bindParam( ':last_name', $this->last_name );
        $stmt->bindParam( ':first_name2', $this->first_name2 );
        $stmt->bindParam( ':last_name2', $this->last_name2 );
        $stmt->bindParam( ':company', $this->company );
        $stmt->bindParam( ':hash', $this->hash );
        $stmt->bindParam( ':mobil', $this->mobil );
        $stmt->bindParam( ':email', $this->email );
        $stmt->bindParam( ':email2', $this->email2 );

        // Execute query
        if( !$stmt->execute() ) {
          error_log( "ERROR: Could create client in database (create_client): " . $stmt->error , 0);
        }

        //sleep half quarter sec
        //usleep(125*1000);

        /////////////////////////////////////////
        // Prepare statement for Correnspondences
        /////////////////////////////////////////
        $query1 = 'INSERT INTO ' . $this->tableCorres . '
                  SET pid = :pid, hid = :hid, flat = :flat, level = :level, isCouncil = :isCouncil, flat_weight = :flatWeight, garage_weight = :garageWeight, oid = :object_id';

        $stmt1 = $this->conn->prepare( $query1 );

        // Clean data
        $this->flat = htmlspecialchars(strip_tags( $this->flat ));
        $this->level = htmlspecialchars(strip_tags( $this->level ));
        $this->isCouncil = htmlspecialchars(strip_tags( $this->isCouncil ));
        $this->flatWeight = htmlspecialchars(strip_tags( $this->flatWeight ));
        $this->garageWeight = htmlspecialchars(strip_tags( $this->garageWeight ));
        $this->object_id = htmlspecialchars(strip_tags( $this->object_id ));

        // Bind data
        $stmt1->bindParam( ':pid', $this->pid );
        $stmt1->bindParam( ':hid', $this->house_id );
        $stmt1->bindParam( ':flat', $this->flat );
        $stmt1->bindParam( ':level', $this->level );
        $stmt1->bindParam( ':isCouncil', $this->isCouncil );
        $stmt1->bindParam( ':flatWeight', $this->flatWeight );
        $stmt1->bindParam( ':garageWeight', $this->garageWeight );
        $stmt1->bindParam( ':object_id', $this->object_id );
        
        // Execute query
        if( !$stmt1->execute() ) {
          error_log( "ERROR: Could not create correspondence (create_client): " . $stmt1->error, 0);
        }

        // commit queries
        $this->conn->commit();

        //sleep for a sec to give db some time to breath
        //sleep(1);
      }
      catch( PDOException $e ) {
        // roll back the transaction if something failed
        $this->conn->rollback();
        
        return false;
      }
      
      return true;
    }


    // Update Client
    public function update_client() {
      // Create query
      $query = 'UPDATE ' . $this->tableClient . '
                SET first_name = :first_name, last_name = :last_name, first_name2 = :first_name2, last_name2 = :last_name2, company = :company, mobil = :mobil, email = :email, email2 = :email2
                WHERE pid = :pid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->pid = htmlspecialchars(strip_tags($this->pid));
      $this->first_name = htmlspecialchars(strip_tags($this->first_name));
      $this->last_name = htmlspecialchars(strip_tags($this->last_name));
      $this->first_name2 = htmlspecialchars(strip_tags($this->first_name2));
      $this->last_name2 = htmlspecialchars(strip_tags($this->last_name2));
      $this->company = htmlspecialchars(strip_tags($this->company));
      $this->mobil = htmlspecialchars(strip_tags($this->mobil));
      $this->email = htmlspecialchars(strip_tags($this->email));
      $this->email2 = htmlspecialchars(strip_tags($this->email2));

      // Bind data
      $stmt->bindParam(':pid', $this->pid);
      $stmt->bindParam(':first_name', $this->first_name);
      $stmt->bindParam(':last_name', $this->last_name);
      $stmt->bindParam(':first_name2', $this->first_name2);
      $stmt->bindParam(':last_name2', $this->last_name2);
      $stmt->bindParam(':company', $this->company);
      $stmt->bindParam(':mobil', $this->mobil);
      $stmt->bindParam(':email', $this->email);
      $stmt->bindParam(':email2', $this->email2);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    public function update_flatweight() {
      // Create query
      $query = 'UPDATE ' . $this->tableCorres . '
                SET flat_weight = :flat_weight, garage_weight = :garage_weight
                WHERE pid = :pid 
                  AND hid = :hid 
                  AND flat = :flat 
                  AND level = :level 
                  AND isCouncil = :isCouncil';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->pid = htmlspecialchars(strip_tags($this->pid));
      $this->house_id = htmlspecialchars(strip_tags( $this->house_id ));
      $this->flat = htmlspecialchars(strip_tags( $this->flat ));
      $this->level = htmlspecialchars(strip_tags( $this->level ));
      $this->isCouncil = htmlspecialchars(strip_tags( $this->isCouncil ));
      $this->flatWeight = htmlspecialchars(strip_tags( $this->flatWeight ));
      $this->garageWeight = htmlspecialchars(strip_tags( $this->garageWeight ));

      // Bind data
      $stmt->bindParam( ':pid', $this->pid );
      $stmt->bindParam( ':hid', $this->house_id );
      $stmt->bindParam( ':flat', $this->flat );
      $stmt->bindParam( ':level', $this->level );
      $stmt->bindParam( ':isCouncil', $this->isCouncil );
      $stmt->bindParam( ':flat_weight', $this->flatWeight );
      $stmt->bindParam( ':garage_weight', $this->garageWeight );

      /* Execute query */
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }

    /**
     * @return bool true if successful, false otherwise.
     */
    public function update_objectid() {
      // Create query
      $query = 'UPDATE ' . $this->tableCorres . '
                SET oid = :oid
                WHERE pid = :pid 
                  AND hid = :hid 
                  AND flat = :flat 
                  AND level = :level
                  AND isCouncil = :isCouncil';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->pid = htmlspecialchars(strip_tags($this->pid));
      $this->house_id = htmlspecialchars(strip_tags( $this->house_id ));
      $this->flat = htmlspecialchars(strip_tags( $this->flat ));
      $this->level = htmlspecialchars(strip_tags( $this->level ));
      $this->isCouncil = htmlspecialchars(strip_tags( $this->isCouncil ));
      $this->object_id = htmlspecialchars(strip_tags( $this->object_id ));

      // Bind data
      $stmt->bindParam( ':pid', $this->pid );
      $stmt->bindParam( ':hid', $this->house_id );
      $stmt->bindParam( ':flat', $this->flat );
      $stmt->bindParam( ':level', $this->level );
      $stmt->bindParam( ':isCouncil', $this->isCouncil );
      $stmt->bindParam( ':oid', $this->object_id );

      /* Execute query */
      if($stmt->execute()) {
        return true;
      }

      return false;
    }


    // Update last used for token
    public function update_last_used_token() {
      // Create query
      $query = 'UPDATE ' . $this->tableDevice . '
                SET last_used=now()
                WHERE token = ?';
     
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->token = htmlspecialchars(strip_tags($this->token));

      // Bind Token
      $stmt->bindParam(1, $this->token);

      // Execute query
      $stmt->execute();
    }


    // Update when device was used
    public function update_last_used_device() {
      // Create query
      $query = 'UPDATE ' . $this->tableDevice . '
                SET last_used=now()
                WHERE deviceId = ?';
     
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->deviceId = htmlspecialchars(strip_tags($this->deviceId));

      // Bind Token
      $stmt->bindParam(1, $this->deviceId);

      // Execute query
      $stmt->execute();
    }


    // Check if token is registered already
    public function is_token_registered() {
      // Create query
      $query = 'SELECT d.token, d.hash, d.deviceId
                FROM ' . $this->tableDevice . ' d
                WHERE d.token = ?';
     
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->token = htmlspecialchars(strip_tags($this->token));

      // Bind Token
      $stmt->bindParam(1, $this->token);

      // Execute query
      $stmt->execute();

      return $stmt;
    }


    // Check if device is registered already
    public function is_device_registered() {
      // Create query
      $query = 'SELECT d.token, d.hash, d.deviceId, d.deviceInfo
                FROM ' . $this->tableDevice . ' d
                WHERE d.deviceId = ?';
     
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->deviceId = htmlspecialchars(strip_tags($this->deviceId));

      // Bind deviceId
      $stmt->bindParam(1, $this->deviceId);

      // Execute query
      $stmt->execute();

      return $stmt;
    }


    // Check if hash is registered already
    public function is_hash_registered() {
      // Create query
      $query = 'SELECT cl.active
                FROM ' . $this->tableClient . ' cl
                WHERE cl.hash = ?';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind hash
      $stmt->bindParam(1, $this->hash);

      // Execute query
      $stmt->execute();

      return $stmt;
    }


    // login
    ////////////////////////////////////////////////////////////////same as is_hash_registered() ??? remove login() when V1 is obsolete
    public function login() {
      // Create query
      $query = 'SELECT cl.active
                FROM ' . $this->tableClient . ' cl
                WHERE cl.hash = ?';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind ID
      $stmt->bindParam(1, $this->hash);

      // Execute query
      $stmt->execute();

      return $stmt;
    }


    //v1 of register
    // Register token + hash in device
    public function register() {
      // Create register query
      $query = 'INSERT INTO ' . $this->tableDevice . ' 
                SET hash = :hash, token = :token';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));
      $this->token = htmlspecialchars(strip_tags($this->token));

      // Bind data
      $stmt->bindParam(':hash', $this->hash);
      $stmt->bindParam(':token', $this->token);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    //v2 of register
    // Register token + hash in device
    public function register_device() {
      // Create register query
      $query = 'INSERT INTO ' . $this->tableDevice . ' 
                SET deviceId = :deviceId, deviceInfo = :deviceInfo, hash = :hash, token = :token';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));
      $this->token = htmlspecialchars(strip_tags($this->token));
      $this->deviceId = htmlspecialchars(strip_tags($this->deviceId));
      $this->deviceInfo = htmlspecialchars(strip_tags($this->deviceInfo));

      // Bind data
      $stmt->bindParam(':hash', $this->hash);
      $stmt->bindParam(':token', $this->deviceId); //workaround, as long as for v1 token needs to be unique (at this point there is no token, see updateToken-method)
      $stmt->bindParam(':deviceId', $this->deviceId);
      $stmt->bindParam(':deviceInfo', $this->deviceInfo);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    //v1 of unregister
    // Unregister / delete token and hash in device table
    public function unregister() {
      // Create query
      $query = 'DELETE FROM ' . $this->tableDevice . ' 
                WHERE token = :token';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->token = htmlspecialchars(strip_tags($this->token));

      // Bind data
      $stmt->bindParam(':token', $this->token);

      // Execute query
      if( $stmt->execute() ) {

        // Get row count
        $num = $stmt->rowCount();
        
        if( $num > 0 ) {
          return true;
        }
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    //v2 of unregister
    // Unregister / delete device (id/hash/token) in device table
    public function unregister_device() {
      // Create query
      $query = 'DELETE FROM ' . $this->tableDevice . ' 
                WHERE deviceId = :deviceId';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->deviceId = htmlspecialchars(strip_tags($this->deviceId));

      // Bind data
      $stmt->bindParam(':deviceId', $this->deviceId);

      // Execute query
      if( $stmt->execute() ) {

        // Get row count
        $num = $stmt->rowCount();
        
        if( $num > 0 ) {
          return true;
        }
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Create deviceId (uuid)
    public function create_device_uuid() {
      // Create query
      $query = 'SELECT UUID()';
     
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      $stmt->execute();

      // Create uuid variable
      $uuid = 0;

      // Execute query
      if( $stmt->execute() ) {

        // Get row count
        $num = $stmt->rowCount();
        
        if( $num > 0 ) {
          $arr = array();
          $i = 0;

          while( $row = $stmt->fetch(PDO::FETCH_ASSOC) ) {
            foreach($row as $key => $value)
              $arr[$i][$key] = $value;
            $i++;
          }

          $uuid = $arr[0]['UUID()'];
        }
      }

      return $uuid;
    }


    // Update token if changed
    public function update_token() {
      // Create query
      $query = 'UPDATE ' . $this->tableDevice . '
                SET token = :token
                WHERE deviceId = :deviceId';
     
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->token = htmlspecialchars(strip_tags($this->token));
      $this->deviceId = htmlspecialchars(strip_tags($this->deviceId));

      // Bind token and deviceId
      $stmt->bindParam(':token', $this->token);
      $stmt->bindParam(':deviceId', $this->deviceId);

      // Execute query
      $stmt->execute();
    }


    //delete device entry by token
    public function delete_device_entry_by_token() {
      // Create query
      $query = 'DELETE FROM ' . $this->tableDevice . ' 
                WHERE token = :token';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->token = htmlspecialchars(strip_tags($this->token));

      // Bind data
      $stmt->bindParam(':token', $this->token);

      // Execute query
      if( $stmt->execute() ) {

        // Get row count
        $num = $stmt->rowCount();
        
        if( $num > 0 ) {
          return true;
        }
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Toggle Active
    public function toggle_active() {
      // Create query
      $query = 'UPDATE ' . $this->tableClient . '
                SET active = :active
                WHERE pid = :pid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      //toggle active
      if( $this->active == 1 )
        $this->active = 0;
      else
        $this->active = 1;

      // Clean data
      $this->active = htmlspecialchars(strip_tags($this->active));
      $this->pid = htmlspecialchars(strip_tags($this->pid));

      // Bind data
      $stmt->bindParam(':active', $this->active);
      $stmt->bindParam(':pid', $this->pid);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Get Houses
    public function get_houses() {
      // Create query
      $query = 'SELECT h.hid, h.name, h.city as house_city
                FROM ' . $this->tableHouse . ' h
                ORDER BY
                  h.hid ASC';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      $stmt->execute();

      return $stmt;
    }


    // Get houses' IDs by hash
    public function get_house_ids() {
      // Create query
      $query = 'SELECT co.hid, co.level
                FROM ' . $this->tableCorres . ' co, ' . $this->tableClient . ' cl
                WHERE cl.hash = ? AND co.pid = cl.pid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind data
      $stmt->bindParam(1, $this->hash);

      // Execute query
      if( $stmt->execute() ) {

        // Get row count
        $num = $stmt->rowCount();

        if( $num > 0 ) {

          $hidArray = array();

          while($row = $stmt->fetch(PDO::FETCH_ASSOC)) {
            extract($row);

            $stack = array($hid, $level);
            array_push( $hidArray, $stack );
          }

          return $hidArray;
        }
      }

      return 0;
    }


    // Checks if house id is available
    public function house_id_available( $houseId ) {
      // Create query
      $query = 'SELECT cl.hash, co.hid
                FROM ' . $this->tableCorres . ' co, ' . $this->tableClient . ' cl
                WHERE cl.hash = ? AND co.hid = ? AND co.pid = cl.pid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));
      $houseId = htmlspecialchars(strip_tags($houseId));

      // Bind data
      $stmt->bindParam(1, $this->hash);
      $stmt->bindParam(2, $houseId);

      // Execute query
      if( $stmt->execute() ) {

        // Get row count
        $num = $stmt->rowCount();

        return ( $num > 0 );
      }

      return false;
    }


    // Get array of available levels for hash and hid
    public function get_level_by_hash_hid() {
      // Create query
      $query = 'SELECT co.level
                FROM ' . $this->tableCorres . ' co, ' . $this->tableClient . ' cl
                WHERE cl.hash = ? AND co.hid = ? AND co.pid = cl.pid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));
      $this->house_id = htmlspecialchars(strip_tags($this->house_id));

      // Bind data
      $stmt->bindParam(1, $this->hash);
      $stmt->bindParam(2, $this->house_id);

      // Execute query
      if( $stmt->execute() ) {

        // Get row count
        $num = $stmt->rowCount();

        if( $num > 0 ) {
          $levelRay = array();

          while( $row = $stmt->fetch(PDO::FETCH_ASSOC) ) {
            extract($row);
            array_push( $levelRay, $level );
          }

          //if array exists
          if( count( $levelRay ) > 0 )
            return $levelRay;
        }
      }

      return false;
    }


    // Get council by hash
    public function get_council() {
      // Create query
      $query = 'SELECT co.isCouncil, co.hid
                FROM ' . $this->tableCorres . ' co JOIN ' . $this->tableClient . ' cl
                ON co.pid = cl.pid
                WHERE cl.hash = ?';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind data
      $stmt->bindParam(1, $this->hash);

      // Execute query
      if( $stmt->execute() ) {

        // Get row count
        $num = $stmt->rowCount();

        if( $num > 0 ) {

          $actIsCouncil = array();
          $i = 0;

          while( $row = $stmt->fetch(PDO::FETCH_ASSOC) ) {
            extract($row);

            if( $isCouncil ) {
              $actIsCouncil[ $i ] = $hid;
              $i += 1;
            }
          }

          //if array exists, client is council at least once
          if( count( $actIsCouncil ) > 0 )
            return $actIsCouncil;
          //client is not council, but client exists
          else
            return 0;
        }
      }

      //client/hash does not exist
      return -1;
    }


    // Create House
    public function create_house_xml($objectID, $immoAddress, $immoZip, $immoCity) {
      // Create query
      //if name is already set -> update, otherwise insert
      $query = 'INSERT INTO ' . $this->tableHouse . ' (hid, name, zip, city)
                VALUES (:hid, :name, :zip, :city) 
                ON DUPLICATE KEY UPDATE hid = :hid1, name = :name1, zip = :zip1, city = :city1';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $objectID = htmlspecialchars(strip_tags($objectID));
      $immoAddress = htmlspecialchars(strip_tags($immoAddress));
      $immoZip = htmlspecialchars(strip_tags($immoZip));
      $immoCity = htmlspecialchars(strip_tags($immoCity));

      // Bind data
      $stmt->bindParam(':hid', $objectID);
      $stmt->bindParam(':name', $immoAddress);
      $stmt->bindParam(':zip', $immoZip);
      $stmt->bindParam(':city', $immoCity);
      $stmt->bindParam(':hid1', $objectID);
      $stmt->bindParam(':name1', $immoAddress);
      $stmt->bindParam(':zip1', $immoZip);
      $stmt->bindParam(':city1', $immoCity);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Update qr_mail_sent for hash
    public function increment_qr_mail_sent() {
      // Create query
      $query = 'UPDATE ' . $this->tableClient . '
                SET qr_mail_sent = qr_mail_sent + 1
                WHERE hash = ?';
     
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind hash
      $stmt->bindParam(1, $this->hash);

      // Execute query
      $stmt->execute();
    }

    // generate sha256 out of client data
    public function make_sha256( $firstName, $surname, $company, $id ) {
    	// Other alogs: "md4", "md5", "sha1", "haval256,3", "tiger160,4" ...

      return hash( "sha256", $firstName.$surname.$company.$id, false );
    }

    public static function toAsciiCode($string) {
      $strlen = strlen($string);
      $charCode = array();
      for ($i = 0; $i < $strlen; $i++) {
          $charCode[] = ord(substr($string, $i, 1));
      }
      $result = json_encode($charCode);
      return $result;
    }

    public function create_object(string $houseId, HouseModel $object) {
      // Create query
      $query = 'INSERT INTO ' . $this->tableObject . ' (oid, hid, title, zip, city, street, number)
                VALUES (:oid, :hid, :title, :zip, :city, :street, :number)
                ON DUPLICATE KEY UPDATE hid = :hid1, title = :title1, zip = :zip1, city = :city1, street = :street1, number = :number1';;

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $oid = htmlspecialchars(strip_tags($object->getHouseId())); // object id...
      $hid = htmlspecialchars(strip_tags($houseId));
      $title = htmlspecialchars(strip_tags($object->getTitle()));
      $zip = htmlspecialchars(strip_tags($object->getZip()));
      $city = htmlspecialchars(strip_tags($object->getCity()));
      $street = htmlspecialchars(strip_tags($object->getStreet()));
      $number = htmlspecialchars(strip_tags($object->getNumber()));

      // Bind data
      $stmt->bindParam(':oid', $oid);
      $stmt->bindParam(':hid', $hid);
      $stmt->bindParam(':title', $title);
      $stmt->bindParam(':zip', $zip);
      $stmt->bindParam(':city', $city);
      $stmt->bindParam(':street', $street);
      $stmt->bindParam(':number', $number);
      $stmt->bindParam(':hid1', $hid);
      $stmt->bindParam(':title1', $title);
      $stmt->bindParam(':zip1', $zip);
      $stmt->bindParam(':city1', $city);
      $stmt->bindParam(':street1', $street);
      $stmt->bindParam(':number1', $number);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      return false;
    }
}