<?php
  require_once('google-api-php-client/vendor/autoload.php');

  class News {

    // DB stuff
    private $conn;
    private $tableNews = 'news';
    private $tableHouse = 'house';

    // News Properties
    public $noOfItems;
    public $offset;

    public $title;
    public $text;
    public $level;
    public $isCouncil;
    public $house_id;

    public $token;

    public $nid;

    public static $GROUPED_NOTIFY_NEWS = "GROUP_NOTIFICATION_NOTIFY_NEWS";

    // Constructor with DB
    public function __construct( $db ) {
      $this->conn = $db;
    }

    // Get news by house id AND LEVEL
    public function get_news_by_id() {

      $queryExtension = "";
      $first = true;
      foreach ($this->level as $value) {
          if( !$first )
            $queryExtension .= " OR ";

          $queryExtension .= "n.level = $value";

          $first = false;
      }

      $localIsCouncil = $this->get_isCouncil_by_id( $this->house_id );
      if( $localIsCouncil )
        $where = 'WHERE n.hid = :hid AND ( n.isCouncil = 1 OR (' . $queryExtension . ') )';
      else
        $where = 'WHERE n.hid = :hid AND ( n.isCouncil = 0 AND (' . $queryExtension . ') )';

      // Create query
      $query = 'SELECT h.name as house_city, n.nid, n.title, n.date, n.text, n.level, n.isCouncil, n.hid
                FROM ' . $this->tableNews . ' n
                LEFT JOIN ' . $this->tableHouse . ' h ON n.hid = h.hid
                ' . $where . '
                ORDER BY n.nid DESC
                LIMIT '.  $this->noOfItems .'
                OFFSET '. $this->offset; //WHERE n.hid = :hid AND ( n.isCouncil <= :isCouncil OR (' . $queryExtension . ') )

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->house_id = htmlspecialchars(strip_tags( $this->house_id ));
      //$localIsCouncil = $this->get_isCouncil_by_id( $this->house_id );

      // Bind data
      $stmt->bindParam(':hid', $this->house_id);
      //$stmt->bindParam(':isCouncil', $localIsCouncil);

      // Execute query
      $stmt->execute();

      return $stmt;
    }

    // Get news by multiple house ids AND LEVEL
    public function get_news_by_ids() {

      // add all news for hid 0, which are created for all people
      $queryExtension = "(n.hid = 0)";

      foreach ($this->house_id as $value) { //$value-array: 0=hid, 1=level
          $queryExtension .= " OR ";

          $localIsCouncil = $this->get_isCouncil_by_id( $value[0] );

          $queryExtension .= "(n.hid = $value[0] AND";

          if( $localIsCouncil )
            $queryExtension .= " (n.isCouncil = 1 OR";
          else
            $queryExtension .= " (n.isCouncil = 0 AND";

          $queryExtension .= " n.level = $value[1]))";
      }

      // Create query
      $query = 'SELECT h.name as house_city, n.nid, n.title, n.date, n.text, n.level, n.isCouncil, n.hid
                FROM ' . $this->tableNews . ' n
                LEFT JOIN ' . $this->tableHouse . ' h ON n.hid = h.hid
                WHERE ' . $queryExtension . '
                ORDER BY n.nid DESC
                LIMIT '.  $this->noOfItems .'
                OFFSET '. $this->offset;

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      //$this->house_id = htmlspecialchars(strip_tags($this->house_id));
      //$this->isCouncil = htmlspecialchars(strip_tags($this->isCouncil));

      // Bind data
      //$stmt->bindParam(':isCouncil', $this->isCouncil);

      // Execute query
      $stmt->execute();

      //if( $stmt->rowCount() < 1 )
      //  error_log( "WARNING: get_news_by_ids issue... Maybe no news available? - query: " . $query, 0);

      return $stmt;
    }

    // Get all news (backend only)
    public function get_all_news() {
        // Create query
        $query = 'SELECT n.nid, n.title, n.date, n.text, n.level, n.isCouncil, n.hid, h.name as house_city
                  FROM ' . $this->tableNews . ' n
                  LEFT JOIN ' . $this->tableHouse . ' h ON n.hid = h.hid
                  ORDER BY n.nid DESC
                  LIMIT '.  $this->noOfItems .'
                  OFFSET '. $this->offset;

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      $stmt->execute();

      return $stmt;
    }

    // Create news
    public function create_news() {
      // Create query
      $query = 'INSERT INTO ' . $this->tableNews . ' 
                SET title = :title, text = :text, level = :level, isCouncil = :isCouncil, hid = :hid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->title = htmlspecialchars(strip_tags($this->title));
      $this->text = htmlspecialchars(strip_tags($this->text));
      $this->level = htmlspecialchars(strip_tags($this->level));
      $this->isCouncil = htmlspecialchars(strip_tags($this->isCouncil));
      $this->house_id = htmlspecialchars(strip_tags($this->house_id));

      // Bind data
      $stmt->bindParam(':title', $this->title);
      $stmt->bindParam(':text', $this->text);
      $stmt->bindParam(':level', $this->level);
      $stmt->bindParam(':isCouncil', $this->isCouncil);
      $stmt->bindParam(':hid', $this->house_id);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //error_log( "ERROR: create_news issue... " . $stmt->error, 0);

      return false;
    }

    // Delete News
    public function delete_news() {
      // Create query
      $query = 'DELETE FROM ' . $this->tableNews . ' 
                WHERE nid = :nid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->nid = htmlspecialchars(strip_tags($this->nid));

      // Bind data
      $stmt->bindParam(':nid', $this->nid);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //error_log( "ERROR: delete_news issue... " . $stmt->error, 0);

      return false;
    }

    // Delete News
    public function delete_multiple_news( $arr ) {

      $inc = 0;
      $nids = "";
      $first = true;
      while( $inc < count($arr) )
      {
        if( !$first )
          $nids .= " OR ";

        $nids .= "nid = " . $arr[$inc];

        $first = false;
        $inc++;
      }

      if( $nids == "" )
          return false;

      // Create query
      $query = 'DELETE FROM ' . $this->tableNews . ' 
                WHERE ' . $nids;

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //error_log( "ERROR: delete_multiple_news issue... " . $stmt->error, 0);

      return false;
    }

    // HELPER function to create OAUTH2 Token
    function getOATHToken()
    {
        $client = new Google_Client();
        try {
            $client->setAuthConfig("/var/www/vhosts/hausverwaltung-kappel.de/httpdocs/app/config/steinle-hausverwaltung-app-firebase-adminsdk-2i76h-297163b4a5.json");
            $client->addScope(Google_Service_FirebaseCloudMessaging::CLOUD_PLATFORM);

            $savedTokenJson = $this->readSavedToken();

            if ($savedTokenJson) {
                // the token exists, set it to the client and check if it's still valid
                $client->setAccessToken($savedTokenJson);
                $accessToken = $savedTokenJson;
                if ($client->isAccessTokenExpired()) {
                    // the token is expired, generate a new token and set it to the client
                    $accessToken = $this->generateToken($client);
                    $client->setAccessToken($accessToken);
                }
            } else {
                // the token doesn't exist, generate a new token and set it to the client
                $accessToken = $this->generateToken($client);
                $client->setAccessToken($accessToken);
            }


            $oauthToken = $accessToken["access_token"];

            return $oauthToken;


        } catch (Google_Exception $e) {}
       return false;
    }

    // HELPER function: Using a simple file to cache and read the toke, can store it in a databse also
    function readSavedToken() {
      $tk = @file_get_contents('token.cache');
      if ($tk) return json_decode($tk, true); else return false;
    }

    // HELPER function: write token
    function writeToken($tk) {
     file_put_contents("token.cache",$tk);
    }

    // HELPER function: generate token
    function generateToken($client)
    {
        $client->fetchAccessTokenWithAssertion();
        $accessToken = $client->getAccessToken();

        $tokenJson = json_encode($accessToken);
        $this->writeToken($tokenJson);

        return $accessToken;
    }

    // Notify News (via topic) - HTTPv1
    public function notify_news() {
        $url = 'https://fcm.googleapis.com/v1/projects/steinle-hausverwaltung-app/messages:send';

        $memberState = "_";
        if( $this->isCouncil == 1)
          $memberState .= "council";
        elseif( $this->level == 1 )
          $memberState .= "owner";
        else
          $memberState .= "renter";

        $topicString = "h" . $this->house_id . $memberState;

        if( $this->house_id == 0 )
          $topicString = "h0all";

        $fields = array (
                'message' => array (
                            "topic" => $topicString,
                            "notification" => array (
                                              "title" => html_entity_decode($this->title),
                                              "body" => html_entity_decode($this->text)
                            ),
                            /*'data' => array (
                                      "story_id" => "story_12345"
                            ),*/
                            "android" => array (
                                        "notification" => array (
                                                          "tag" => self::$GROUPED_NOTIFY_NEWS
                                        )
                            ),
                            //https://www.npmjs.com/package/cordova-plugin-firebasex#ios-notifications
                            "apns" => array (
                                    "payload" => array (
                                                "aps" => array (
                                                        "alert" => array (
                                                                          "title" => html_entity_decode($this->title),
                                                                          "body" => html_entity_decode($this->text)
                                                        ),
                                                        //"sound" => "default",
                                                        "thread-id" => self::$GROUPED_NOTIFY_NEWS
                                                )
                                    )
                            )
                )
        );

        $headers = array (
                'Authorization: Bearer ' . $this->getOATHToken(),
                'Content-Type: application/json'
        );

        $ch = curl_init ();
        curl_setopt ( $ch, CURLOPT_URL, $url );
        curl_setopt ( $ch, CURLOPT_POST, true );
        curl_setopt ( $ch, CURLOPT_HTTPHEADER, $headers );
        curl_setopt ( $ch, CURLOPT_RETURNTRANSFER, true );
        curl_setopt(  $ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt ( $ch, CURLOPT_POSTFIELDS, json_encode($fields) ); //urldecode

        $result = curl_exec( $ch );
        //error_log( "INFO: notify_news_v1 issue... " . $result, 0);
        curl_close ( $ch );

        $cur_message = json_decode($result);
        if( isset($cur_message->error) ) {
          error_log( "ERROR: notify_news_v1 issue... " . $result, 0);
          return false;
        }
        else
            return true;
    }

    //check if client isCouncil for house id
    private function get_isCouncil_by_id( $hid ) {
      $arr = $this->isCouncil;

      if( is_array($arr) && count($arr) < 1 )
        return 0;

      if( is_array($arr) ) {
        foreach ($arr as &$value) {
          if( $value == $hid )
            return 1;
        }
      }

      return 0;
    }

  }
