<?php 
  class Poll {
    
    // DB stuff
    private $conn;
    private $tablePoll = 'poll';
    private $tableAnswer = 'poll_answer';
    private $tableSettings = 'poll_settings';
    private $tableProxy = 'poll_proxy';
    private $tableClient = 'client';
    private $tableCorres = 'correspondence';
    private $tableHouse = 'house';
    private $tableObject = 'object';

    // Poll Properties
    public $phash; 
    public $hid; 
    public $active;  
    public $pass;
    public $weight_settings = null;
    public $object_settings = null;
    public $absentee_hash;

    // Decision Properties
    public $decision_1;  
    public $decision_2;  
    public $decision_3;  
    public $decision_4;  
    public $decision_5;  
    public $decision_6;  
    public $decision_7;  
    public $decision_8;  
    public $decision_9;  
    public $decision_10; 
    public $decision_11; 
    public $decision_12; 
    public $decision_13; 
    public $decision_14; 
    public $decision_15; 
    public $decision_16; 
    public $decision_17; 
    public $decision_18; 
    public $decision_19; 
    public $decision_20;
    public $decision_21; 
    public $decision_22; 
    public $decision_23; 
    public $decision_24; 
    public $decision_25; 
    public $decision_26; 
    public $decision_27; 
    public $decision_28; 
    public $decision_29; 
    public $decision_30;

    // Further
    private $max_decisions = 30;

    // Answer Properties
    public $aid;
    public $hash;
    public $answer_1;  
    public $answer_2;  
    public $answer_3; 
    public $answer_4;  
    public $answer_5;
    public $answer_6;
    public $answer_7;  
    public $answer_8;  
    public $answer_9;  
    public $answer_10; 
    public $answer_11; 
    public $answer_12; 
    public $answer_13; 
    public $answer_14; 
    public $answer_15; 
    public $answer_16; 
    public $answer_17; 
    public $answer_18; 
    public $answer_19; 
    public $answer_20;
    public $answer_21; 
    public $answer_22; 
    public $answer_23; 
    public $answer_24; 
    public $answer_25; 
    public $answer_26; 
    public $answer_27; 
    public $answer_28; 
    public $answer_29; 
    public $answer_30;

    // Constructor with DB
    public function __construct( $db ) {
      $this->conn = $db;
    }


    // Check if a poll is active
    public function is_poll_active() {
      // Create query
      $query = 'SELECT p.active, p.phash, p.name
                FROM ' . $this->tablePoll . ' p
                WHERE p.hid = ? AND p.active = 1';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->hid = htmlspecialchars(strip_tags($this->hid));

      // Bind ID
      $stmt->bindParam( 1, $this->hid );

      // Execute query
      if( $stmt->execute() ) {

        $row = $stmt->fetch(PDO::FETCH_ASSOC);

        if( isset($row['phash']) )
          return array( "phash" => $row['phash'], "name" => $row['name'] );
      }

      return 0;
    }


    // Check if ANY poll is active
    public function is_any_poll_active() {
      // Create query
      $query = 'SELECT p.active, p.phash
                FROM ' . $this->tablePoll . ' p
                WHERE p.active = 1 AND p.phash != ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind ID
      $stmt->bindParam( 1, $this->phash );

      // Execute query
      if( $stmt->execute() ) {

        if( $stmt->rowCount() > 0 )
          return 1;
      }

      return 0;
    }


    // When user wants to login with password
    public function verify() {
      // Create query
      $query = 'SELECT p.active, p.pass
                FROM ' . $this->tablePoll . ' p
                WHERE p.phash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->pass = htmlspecialchars(strip_tags($this->pass));

      // Bind ID
      $stmt->bindParam( 1, $this->phash );

      // Execute query
      if( $stmt->execute() ) {

          $row = $stmt->fetch(PDO::FETCH_ASSOC);
          //if( isset($row['active']) && !empty( $row['active']) )
          if( !isset( $row['active'] ) || $row['active'] === 0 )
            return 2;

          if( isset( $row['pass']) ) {
            if( $row['pass'] == $this->pass )
              return 1;
            else 
              return 0;
          }
      }

      return 3;
    }


    // Get decision by index
    public function get_decision_by_index( $index ) {

      if( $index <= 0 || $index > $this->max_decisions )
        return null;

      $column = "decision_" . $index;

      // Create query
      $query = 'SELECT p. '.$column.'
                FROM ' . $this->tablePoll . ' p
                WHERE p.phash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind ID
      $stmt->bindParam( 1, $this->phash );

      // Execute query
      if( $stmt->execute() ) {

          $row = $stmt->fetch(PDO::FETCH_ASSOC);
          return $row[ $column ];
      }

      return null;
    }


    // Get number of decisions
    public function get_no_of_decisions() {
      // Create query
      $query = 'SELECT p.no_of_decisions
                FROM ' . $this->tablePoll . ' p
                WHERE p.phash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind ID
      $stmt->bindParam( 1, $this->phash );

      // Execute query
      if( $stmt->execute() ) {

          $row = $stmt->fetch(PDO::FETCH_ASSOC);
          return $row['no_of_decisions'];
      }

      return 0;
    }


    // Get current active decision
    public function get_current_active_decision() {
      // Create query
      $query = 'SELECT p.active_decision
                FROM ' . $this->tablePoll . ' p
                WHERE p.phash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind ID
      $stmt->bindParam( 1, $this->phash );

      // Execute query
      if( $stmt->execute() ) {

          $row = $stmt->fetch(PDO::FETCH_ASSOC);
          return $row['active_decision'];
      }

      return 0;
    }


    // Check if user entry exists in poll_answer (if user registered or not)
    public function does_answer_entry_exist() {
      // Create query
      $query = 'SELECT p.answered_decisions
                FROM ' . $this->tableAnswer . ' p
                WHERE p.phash = ? AND p.hash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind ID
      $stmt->bindParam( 1, $this->phash );
      $stmt->bindParam( 2, $this->hash );

      //debug purpose
      //error_log( "DEBUG: poll - does_answer_entry_exist() - phash: " . $this->phash . " - hash: " . $this->hash, 0);

      // Execute query
      if( $stmt->execute() ) {

        return $stmt->rowCount();

      }

      return 0;
    }


    // Create user entry in answer table (register user for poll)
    public function create_answer_entry() {
      // Create query
      $query = 'INSERT INTO ' . $this->tableAnswer . '
                SET phash = :phash, hash = :hash';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind ID
      $stmt->bindParam( ':phash', $this->phash );
      $stmt->bindParam( ':hash', $this->hash );

      // Execute query
      if( $stmt->execute() ) {

          return 1;
      }

      return 0;
    }


    // Get index of last answered decision (by default 0, means 1 is the next/first decision)
    public function get_last_answered_id() {
      // Create query
      $query = 'SELECT p.answered_decisions
                FROM ' . $this->tableAnswer . ' p
                WHERE p.phash = ? AND p.hash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind ID
      $stmt->bindParam( 1, $this->phash );
      $stmt->bindParam( 2, $this->hash );

      // Execute query
      if( $stmt->execute() ) {

          $row = $stmt->fetch(PDO::FETCH_ASSOC);
          return $row['answered_decisions'];
      }

      return -1;
    }


    // Get how many decisions are already answered for a poll (all users)
    public function get_number_of_answered_decisions() {
      $inc = 1;
      $columns = "";

      while( $inc <= $this->max_decisions )
      {
        $columns .= "Max(answer_" . $inc . ")";

        $inc++;
        
        //check if there is a next
        if( $inc <= $this->max_decisions )
          $columns .= ", ";
      }

      // Create query
      $query = 'SELECT ' . $columns . '
                FROM ' . $this->tableAnswer . ' p
                WHERE p.phash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind ID
      $stmt->bindParam( 1, $this->phash );

      // Execute query
      if( $stmt->execute() ) {

        $row = $stmt->fetch(PDO::FETCH_ASSOC);
        $count = 0;

        for( $i=1; $i <= $this->max_decisions; $i++ ){
          if( $row['Max(answer_'.$i.')'] > 0 ) $count++;
        }

        return $count;
      }

      return -1;
    }


    // Get answer by index
    public function is_answered( $index ) {

      if( $index <= 0 || $index > $this->max_decisions )
        return null;

      $column = "answer_" . $index;

      // Create query
      $query = 'SELECT '.$column.'
                FROM ' . $this->tableAnswer . ' p
                WHERE p.phash = ? AND p.hash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind ID
      $stmt->bindParam( 1, $this->phash );
      $stmt->bindParam( 2, $this->hash );

      // Execute query
      if( $stmt->execute() ) {
        if( $stmt->rowCount() > 0 )
        {
          $row = $stmt->fetch(PDO::FETCH_ASSOC);
          if( isset($row[ $column ]) ) {
            return $row[ $column ];
          }
        }
      }

      return 0;
    }


    // Check if all decisions are answered for user
    public function is_all_answered() {

      $inc = 1;
      $first = true;
      $columns = "";

      while( $inc <= $this->get_no_of_decisions() )
      {
        if( !$first )
          $columns .= " AND ";

        $columns .= "answer_" . $inc . "<> 0";

        $inc++;
        $first = false;
      }

      // Create query
      $query = 'SELECT a.hash
                FROM ' . $this->tableAnswer . ' a
                WHERE phash = :phash AND hash = :hash AND ' . $columns;
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);
      $stmt->bindParam(':hash', $this->hash);

      // Execute query
      if( $stmt->execute() ) {

        if( $stmt->rowCount() > 0 )
          return true;

      }

      return false;
    }


    // Get index of last answered decision
    public function inc_last_answered_id() {
      // Create query
      $query = 'UPDATE ' . $this->tableAnswer . ' p
                SET answered_decisions = answered_decisions+?
                WHERE p.phash = ? AND p.hash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind ID
      $increment = 1;
      $stmt->bindParam( 1, $increment );
      $stmt->bindParam( 2, $this->phash );
      $stmt->bindParam( 3, $this->hash );

      // Execute query
      if( $stmt->execute() ) {
          return 1;
      }

      return 0;
    }


    // Get next decision that is not answered yet
    public function get_next_decision() {
      $answer_id = $this->get_current_active_decision();//$this->get_last_answered_id();

      //if there is no active decision
      if( $answer_id < 1 )
        return array( 888, "-" );

      if( $answer_id < 0 )
        return null;

      //return if decision is already answered (1-3) otherwise 0
      $answer = $this->is_answered( $answer_id );
      
      //last decision already reached
      /*if( $this->is_all_answered() )
        return array( 999, "-" );*/

      //by user
      //$answered_decisions = $this->get_last_answered_id();
      //by all users
      $answered_decisions = $this->get_number_of_answered_decisions();

/*
      //last decision already reached
      if( $answer_id + 1 > $this->get_no_of_decisions() )
        return array( 999, "-" );

      //check if next decision is current active one
      if( $answer_id + 1 != $this->get_current_active_decision() )
        return array( 888, "-" );
      
      $answer_id += 1;
*/

      return array( $answer_id, $this->get_decision_by_index( $answer_id ), $answer, $answered_decisions );
    }


    // Sends an answer to a specific decision
    public function send_answer_by_index( $index, $answer ) {

      if( $index <= 0 || $index > $this->max_decisions )
        return null;

      $column = "answer_" . $index;

      // Create query
      $query = 'UPDATE ' . $this->tableAnswer . ' p
                SET '.$column.' = ?
                WHERE p.phash = ? AND p.hash = ?';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $answer = htmlspecialchars(strip_tags($answer));
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind ID
      $stmt->bindParam( 1, $answer );
      $stmt->bindParam( 2, $this->phash );
      $stmt->bindParam( 3, $this->hash );

      // Execute query
      if( $stmt->execute() ) {

          return 1;
      }

      return 0;
    }


    // Sends just the next answer
    public function send_answer( $answer ) {
      if( $answer === 0 )
        return null;

      //$answer_id = $this->get_last_answered_id();
      $answer_id = $this->get_current_active_decision();

      if( $this->is_answered( $answer_id) > 0 )
        return $answer_id;

      if( $answer_id < 0 || $answer_id > $this->max_decisions )
        return null;

      //$answer_id += 1;

      if( $this->send_answer_by_index( $answer_id, $answer ) === 0 )
        return null;

      if( $this->inc_last_answered_id() === 0 )
        return null;

      return $answer_id;
    }


    // Create pash (SAME FUNCTION AS FOR CREATION OF deviceId (uuid))
    public function create_poll_uuid() {
      // Create query
      $query = 'SELECT UUID()';
     
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      $stmt->execute();

      // Create uuid variable
      $uuid = 0;

      // Execute query
      if( $stmt->execute() ) {

        // Get row count
        $num = $stmt->rowCount();
        
        if( $num > 0 ) {
          $arr = array();
          $i = 0;

          while( $row = $stmt->fetch(PDO::FETCH_ASSOC) ) {
            foreach($row as $key => $value)
              $arr[$i][$key] = $value;
            $i++;
          }

          $uuid = $arr[0]['UUID()'];
        }
      }

      return $uuid;
    }


    // Get a list (array) of abseentes
    public function get_absentee_list_by_agenthash( $hash ) {
      // Create query
      $query = 'SELECT p.pid_absentee, c.hash, c.first_name, c.last_name, c.company
                FROM ' . $this->tableProxy . ' p
                LEFT JOIN ' . $this->tableClient . ' c ON p.pid_absentee = c.pid
                WHERE p.phash = :phash AND p.pid_agent = (SELECT pid FROM ' . $this->tableClient . ' WHERE hash = "' . $hash . '" )';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);

      // Execute query
      if( $stmt->execute() ) {

        if( $stmt->rowCount() > 0 ) {

            $abs_arr = array();
            $index = 0;

            while( $row = $stmt->fetch(PDO::FETCH_ASSOC) ) {
              extract( $row );

              $abs_arr[ $index ]['hash'] = $hash;
              $abs_arr[ $index ]['name'] = $first_name . " " . $last_name . " " . $company;

              $index++;
            }

            return $abs_arr;
        }

      }

      return null;
    }


    // Check if user has the rights to vote for decision
    public function is_user_eligible_to_vote( $decision, $weightStr ) {

      /* manual:
         check in settings if oids/weights are available
         -> if not -> everyone can access
         -> if yes -> check if user is allowed to access
                      -> if same oid as user and corresponding weight >0 -> user can access
                      -> if not -> no access, read only */

      //if poll_is_object_weight_set_by_decision retuns false -> everyone can vote (0)
      $everyone_can_vote = !$this->poll_is_object_weight_set_by_decision( $decision );

      if( $everyone_can_vote ) 
        return true;

      if( $this->is_user_eligible_according_to_property_type( $decision, $weightStr ) )
        return true;

      return false;
    }


    // Function returns if there are oids or weights for poll via decision available
    public function poll_is_object_weight_set_by_decision( $decision ) {
      // Create query
      $query = 'SELECT s.value 
                FROM ' . $this->tableSettings . ' s 
                WHERE phash = :phash AND decision = '.$decision.' AND (setting = "object" OR setting = "weight_type")';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);

      // Execute query
      if( $stmt->execute() ) {

          if( $stmt->rowCount() > 0 ) 
            return true;
      }

      return false;
    }


    // Check if user is eligible to vote related to flat and garage and oids
    public function is_user_eligible_according_to_property_type( $decision, $weightStr ) {

      $weightQuery = "";

      switch ($weightStr) {
          case "garage":
              $weightQuery = "AND co.garage_weight > 0 ";
              break;
          case "flat":
              $weightQuery = "AND co.flat_weight > 0 ";
              break;
          default: //"default"
              $weightQuery = "";
      }

      // Create query
      $query = 'SELECT DISTINCT co.cid, co.flat_weight, co.garage_weight, co.hid
                FROM ' . $this->tablePoll . ' p, ' . $this->tableSettings . ' s, ' . $this->tableCorres . ' co, ' . $this->tableClient . ' cl 
                WHERE p.phash = :phash
                AND cl.hash = :hash
                AND p.hid = co.hid 
                AND co.pid = cl.pid 
                ' . $weightQuery . '
                AND ((s.phash = p.phash AND s.decision = '.$decision.' AND s.setting = "object" AND s.value = co.oid) 
                     OR 
                     NOT EXISTS (SELECT ps.value 
                             FROM ' . $this->tableSettings . ' ps 
                                 WHERE ps.phash = :phash AND ps.decision = '.$decision.' AND ps.setting = "object")
                    )';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);
      $stmt->bindParam(':hash', $this->hash);

      // Execute query
      if( $stmt->execute() ) {

          if( $stmt->rowCount() > 0 ) 
            return true;
      }
      
      return false;
    }


    // Check if user oid is available for current poll/decision
    public function is_user_object_available( $decision ) {
      // Create query
      $query = 'SELECT b.oid 
                FROM (SELECT s.value 
                      FROM ' . $this->tableSettings . ' s 
                      WHERE phash = :phash AND decision = '.$decision.' AND setting = "object") a, 
                     (SELECT co.oid 
                      FROM ' . $this->tableCorres . ' co, ' . $this->tableClient . ' cl 
                      WHERE cl.hash = :hash AND cl.pid = co.pid) b
              WHERE a.value = b.oid';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->hash = htmlspecialchars(strip_tags($this->hash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);
      $stmt->bindParam(':hash', $this->hash);

      // Execute query
      if( $stmt->execute() ) {

          if( $stmt->rowCount() > 0 ) 
            return true;
      }
      
      return false;
    }




    /********************** SERVER SIDE ********************************/




    // Get a list of all polls, ordered by active->date
    public function get_all_polls() {
      // Create query
      $query = 'SELECT p.phash, p.name, p.date, p.hid, h.city as house_city, h.name as house_name, p.active, p.pass, p.no_of_decisions, p.decision_1,
                p.decision_2, p.decision_3, p.decision_4, p.decision_5, p.decision_6, p.decision_7, 
                p.decision_8, p.decision_9, p.decision_10, p.decision_11, p.decision_12, p.decision_13, 
                p.decision_14, p.decision_15, p.decision_16, p.decision_17, p.decision_18, p.decision_19, 
                p.decision_20, p.decision_21, p.decision_22, p.decision_23, p.decision_24, p.decision_25, 
                p.decision_26, p.decision_27, p.decision_28, p.decision_29, p.decision_30
                FROM ' . $this->tablePoll . ' p
                LEFT JOIN ' . $this->tableHouse . ' h ON p.hid = h.hid
                ORDER BY active DESC, date DESC';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      if( $stmt->execute() ) {

        if( $stmt->rowCount() > 0 )
          return $stmt;

      }

      return null;
    }


    // Create a new poll with decisions, password, date, object ...
    public function create_new_poll() {
      // Create query
      $query = 'INSERT INTO ' . $this->tablePoll . ' 
                SET phash = :phash, name = :name, hid = :hid, active = :active, pass = :pass,
                no_of_decisions = :no_of_decisions, decision_1 = :decision_1, decision_2 = :decision_2,
                decision_3 = :decision_3, decision_4 = :decision_4, decision_5 = :decision_5, 
                decision_6 = :decision_6, decision_7 = :decision_7, decision_8 = :decision_8, 
                decision_9 = :decision_9, decision_10 = :decision_10, decision_11 = :decision_11,
                decision_12 = :decision_12, decision_13 = :decision_13, decision_14 = :decision_14, 
                decision_15 = :decision_15, decision_16 = :decision_16, decision_17 = :decision_17,
                decision_18 = :decision_18, decision_19 = :decision_19, decision_20 = :decision_20, 
                decision_21 = :decision_21, decision_22 = :decision_22, decision_23 = :decision_23, 
                decision_24 = :decision_24, decision_25 = :decision_25, decision_26 = :decision_26, 
                decision_27 = :decision_27, decision_28 = :decision_28, decision_29 = :decision_29, 
                decision_30 = :decision_30';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->name = htmlspecialchars(strip_tags($this->name));
      $this->hid = htmlspecialchars(strip_tags($this->hid));
      $this->active = htmlspecialchars(strip_tags($this->active));
      $this->pass = htmlspecialchars(strip_tags($this->pass));
      $this->no_of_decisions = htmlspecialchars(strip_tags($this->no_of_decisions));
      $this->decision_1 = htmlspecialchars(strip_tags($this->decision_1));
      $this->decision_2 = htmlspecialchars(strip_tags($this->decision_2));
      $this->decision_3 = htmlspecialchars(strip_tags($this->decision_3));
      $this->decision_4 = htmlspecialchars(strip_tags($this->decision_4));
      $this->decision_5 = htmlspecialchars(strip_tags($this->decision_5));
      $this->decision_6 = htmlspecialchars(strip_tags($this->decision_6));
      $this->decision_7 = htmlspecialchars(strip_tags($this->decision_7));
      $this->decision_8 = htmlspecialchars(strip_tags($this->decision_8));
      $this->decision_9 = htmlspecialchars(strip_tags($this->decision_9));
      $this->decision_10 = htmlspecialchars(strip_tags($this->decision_10));
      $this->decision_11 = htmlspecialchars(strip_tags($this->decision_11));
      $this->decision_12 = htmlspecialchars(strip_tags($this->decision_12));
      $this->decision_13 = htmlspecialchars(strip_tags($this->decision_13));
      $this->decision_14 = htmlspecialchars(strip_tags($this->decision_14));
      $this->decision_15 = htmlspecialchars(strip_tags($this->decision_15));
      $this->decision_16 = htmlspecialchars(strip_tags($this->decision_16));
      $this->decision_17 = htmlspecialchars(strip_tags($this->decision_17));
      $this->decision_18 = htmlspecialchars(strip_tags($this->decision_18));
      $this->decision_19 = htmlspecialchars(strip_tags($this->decision_19));
      $this->decision_20 = htmlspecialchars(strip_tags($this->decision_20));
      $this->decision_21 = htmlspecialchars(strip_tags($this->decision_21));
      $this->decision_22 = htmlspecialchars(strip_tags($this->decision_22));
      $this->decision_23 = htmlspecialchars(strip_tags($this->decision_23));
      $this->decision_24 = htmlspecialchars(strip_tags($this->decision_24));
      $this->decision_25 = htmlspecialchars(strip_tags($this->decision_25));
      $this->decision_26 = htmlspecialchars(strip_tags($this->decision_26));
      $this->decision_27 = htmlspecialchars(strip_tags($this->decision_27));
      $this->decision_28 = htmlspecialchars(strip_tags($this->decision_28));
      $this->decision_29 = htmlspecialchars(strip_tags($this->decision_29));
      $this->decision_30 = htmlspecialchars(strip_tags($this->decision_30));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);
      $stmt->bindParam(':name', $this->name);
      $stmt->bindParam(':hid', $this->hid);
      $stmt->bindParam(':active', $this->active);
      $stmt->bindParam(':pass', $this->pass);
      $stmt->bindParam(':no_of_decisions', $this->no_of_decisions);
      $stmt->bindParam(':decision_1', $this->decision_1);
      $stmt->bindParam(':decision_2', $this->decision_2);
      $stmt->bindParam(':decision_3', $this->decision_3);
      $stmt->bindParam(':decision_4', $this->decision_4);
      $stmt->bindParam(':decision_5', $this->decision_5);
      $stmt->bindParam(':decision_6', $this->decision_6);
      $stmt->bindParam(':decision_7', $this->decision_7);
      $stmt->bindParam(':decision_8', $this->decision_8);
      $stmt->bindParam(':decision_9', $this->decision_9);
      $stmt->bindParam(':decision_10', $this->decision_10);
      $stmt->bindParam(':decision_11', $this->decision_11);
      $stmt->bindParam(':decision_12', $this->decision_12);
      $stmt->bindParam(':decision_13', $this->decision_13);
      $stmt->bindParam(':decision_14', $this->decision_14);
      $stmt->bindParam(':decision_15', $this->decision_15);
      $stmt->bindParam(':decision_16', $this->decision_16);
      $stmt->bindParam(':decision_17', $this->decision_17);
      $stmt->bindParam(':decision_18', $this->decision_18);
      $stmt->bindParam(':decision_19', $this->decision_19);
      $stmt->bindParam(':decision_20', $this->decision_20);
      $stmt->bindParam(':decision_21', $this->decision_21);
      $stmt->bindParam(':decision_22', $this->decision_22);
      $stmt->bindParam(':decision_23', $this->decision_23);
      $stmt->bindParam(':decision_24', $this->decision_24);
      $stmt->bindParam(':decision_25', $this->decision_25);
      $stmt->bindParam(':decision_26', $this->decision_26);
      $stmt->bindParam(':decision_27', $this->decision_27);
      $stmt->bindParam(':decision_28', $this->decision_28);
      $stmt->bindParam(':decision_29', $this->decision_29);
      $stmt->bindParam(':decision_30', $this->decision_30);

      // Execute query
      if($stmt->execute()) {
        return $this->create_poll_settings_weight() && $this->create_poll_settings_object();
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Create a new poll setting for weight
    public function create_poll_settings_weight() {
      if( !is_array($this->weight_settings) || count($this->weight_settings) == 0 )
        return true;

      $query = "";
      for( $i=0; $i<count($this->weight_settings); $i++ ) {

        // Clean data
        $p = htmlspecialchars(strip_tags($this->phash));
        $d = htmlspecialchars(strip_tags($this->weight_settings[$i]["decision_no"]));
        $s = htmlspecialchars(strip_tags($this->weight_settings[$i]["setting"]));
        $v = htmlspecialchars(strip_tags($this->weight_settings[$i]["value"]));

        //if no setting is set, it makes no sense to proceed with this item
        if( $s == "" )
          continue;

        // Create query
        $query .= 'INSERT INTO ' . $this->tableSettings . ' (phash, decision, setting, value) 
                  VALUES ( "'.$p.'", '.$d.', "'.$s.'", "'.$v.'" )';
        
        if( $i+1 < count($this->weight_settings) )
          $query .= ";";
      }

      if( $query == "" )
        return true;

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Create a new poll setting for object
    public function create_poll_settings_object() {
      if( !is_array($this->object_settings) || count($this->object_settings) == 0 )
        return true;

      $query = "";
      for( $i=0; $i<count($this->object_settings); $i++ ) {

        // Clean data
        $p = htmlspecialchars(strip_tags($this->phash));
        $d = htmlspecialchars(strip_tags($this->object_settings[$i]["decision_no"]));
        $s = htmlspecialchars(strip_tags($this->object_settings[$i]["setting"]));
        $v = htmlspecialchars(strip_tags($this->object_settings[$i]["value"]));

        //if no setting is set, it makes no sense to proceed with this item
        if( $s == "" )
          continue;

        // Create query
        $query .= 'INSERT INTO ' . $this->tableSettings . ' (phash, decision, setting, value) 
                  VALUES ( "'.$p.'", '.$d.', "'.$s.'", "'.$v.'" )';
      
        if( $i+1 < count($this->object_settings) )
          $query .= ";";
      }

      if( $query == "" )
        return true;

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Get all poll settings for weight
    public function get_poll_settings_weight_by_phash() {
      // Create query
      $query = 'SELECT decision, value
                FROM ' . $this->tableSettings . '
                WHERE phash = :phash AND setting = "weight_type"';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);

      // Execute query
      if( $stmt->execute() ) {

        if( $stmt->rowCount() > 0 && $stmt != null  ) {

            //create array
            $this->weight_settings = array();

            while( $row = $stmt->fetch(PDO::FETCH_ASSOC) ) {
              extract( $row );

              //create item/row
              $weight_item = array(
                'decision_no' => (int)$decision,
                'value' => html_entity_decode($value)
              );

              // Push data set to array
              array_push( $this->weight_settings, $weight_item );
            }

          return $this->weight_settings;
          
        }
      }

      return null;
    }


    // Get all poll settings for object
    public function get_poll_settings_object_by_phash() {
      // Create query
      $query = 'SELECT s.decision, s.value, o.title
                FROM ' . $this->tableSettings . ' s
                LEFT JOIN ' . $this->tableObject . ' o ON o.oid = s.value
                WHERE phash = :phash AND setting = "object"';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);

      // Execute query
      if( $stmt->execute() ) {

        if( $stmt->rowCount() > 0 && $stmt != null  ) {

            //create array
            $this->object_settings = array();

            while( $row = $stmt->fetch(PDO::FETCH_ASSOC) ) {
              extract( $row );

              //create item/row
              $obj_item = array(
                'decision_no' => (int)$decision,
                'value' => html_entity_decode($value),
                'title' => html_entity_decode($title)
              );

              // Push data set to array
              array_push( $this->object_settings, $obj_item );
            }

          return $this->object_settings;
          
        }
      }

      return null;
    }


    // Get the poll settings of weight type for a specific decision
    public function get_poll_setting_weight_by_phash_decision( $decision ) {
      if( $decision === 0 )
        return null;

      // Create query
      $query = 'SELECT value
                FROM ' . $this->tableSettings . '
                WHERE phash = :phash AND decision = '.$decision.' AND setting = "weight_type"';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);

      // Execute query
      if( $stmt->execute() ) {

          if( $stmt->rowCount() > 0 ) {
            $row = $stmt->fetch(PDO::FETCH_ASSOC);
            return $row['value']; //garage or flat
          }
          else {
            return "default";
          }
      }

      return null;
    }


    // Get the poll settings of object type for a specific decision
    public function get_poll_setting_object_by_phash_decision( $decision ) {
      if( $decision === 0 )
        return null;

      // Create query
      $query = 'SELECT value
                FROM ' . $this->tableSettings . '
                WHERE phash = :phash AND decision = '.$decision.' AND setting = "object"';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);

      // Execute query
      if( $stmt->execute() ) {

          if( $stmt->rowCount() > 0 ) {
            //old
            //$row = $stmt->fetch(PDO::FETCH_ASSOC);
            //return $row['value']; //WEG Blub 1a

            $obj_arr = array();
            $index = 0;

            while( $row = $stmt->fetch(PDO::FETCH_ASSOC) ) {
              extract( $row );

              $obj_arr[ $index ] = $value;

              $index++;
            }

            return $obj_arr;
          }
          else {
            return "";
          }
      }

      return null;
    }


    // Update an existing poll with decisions, password, date, object ...
    public function edit_existing_poll() {
      // Create query
      $query = 'UPDATE ' . $this->tablePoll . ' 
                SET name = :name,pass = :pass,
                no_of_decisions = :no_of_decisions, decision_1 = :decision_1, decision_2 = :decision_2,
                decision_3 = :decision_3, decision_4 = :decision_4, decision_5 = :decision_5, 
                decision_6 = :decision_6, decision_7 = :decision_7, decision_8 = :decision_8, 
                decision_9 = :decision_9, decision_10 = :decision_10, decision_11 = :decision_11,
                decision_12 = :decision_12, decision_13 = :decision_13, decision_14 = :decision_14, 
                decision_15 = :decision_15, decision_16 = :decision_16, decision_17 = :decision_17,
                decision_18 = :decision_18, decision_19 = :decision_19, decision_20 = :decision_20, 
                decision_21 = :decision_21, decision_22 = :decision_22, decision_23 = :decision_23, 
                decision_24 = :decision_24, decision_25 = :decision_25, decision_26 = :decision_26, 
                decision_27 = :decision_27, decision_28 = :decision_28, decision_29 = :decision_29, 
                decision_30 = :decision_30
                WHERE phash = :phash';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->name = htmlspecialchars(strip_tags($this->name));
      $this->active = htmlspecialchars(strip_tags($this->active));
      $this->pass = htmlspecialchars(strip_tags($this->pass));
      $this->no_of_decisions = htmlspecialchars(strip_tags($this->no_of_decisions));
      $this->decision_1 = htmlspecialchars(strip_tags($this->decision_1));
      $this->decision_2 = htmlspecialchars(strip_tags($this->decision_2));
      $this->decision_3 = htmlspecialchars(strip_tags($this->decision_3));
      $this->decision_4 = htmlspecialchars(strip_tags($this->decision_4));
      $this->decision_5 = htmlspecialchars(strip_tags($this->decision_5));
      $this->decision_6 = htmlspecialchars(strip_tags($this->decision_6));
      $this->decision_7 = htmlspecialchars(strip_tags($this->decision_7));
      $this->decision_8 = htmlspecialchars(strip_tags($this->decision_8));
      $this->decision_9 = htmlspecialchars(strip_tags($this->decision_9));
      $this->decision_10 = htmlspecialchars(strip_tags($this->decision_10));
      $this->decision_11 = htmlspecialchars(strip_tags($this->decision_11));
      $this->decision_12 = htmlspecialchars(strip_tags($this->decision_12));
      $this->decision_13 = htmlspecialchars(strip_tags($this->decision_13));
      $this->decision_14 = htmlspecialchars(strip_tags($this->decision_14));
      $this->decision_15 = htmlspecialchars(strip_tags($this->decision_15));
      $this->decision_16 = htmlspecialchars(strip_tags($this->decision_16));
      $this->decision_17 = htmlspecialchars(strip_tags($this->decision_17));
      $this->decision_18 = htmlspecialchars(strip_tags($this->decision_18));
      $this->decision_19 = htmlspecialchars(strip_tags($this->decision_19));
      $this->decision_20 = htmlspecialchars(strip_tags($this->decision_20));
      $this->decision_21 = htmlspecialchars(strip_tags($this->decision_21));
      $this->decision_22 = htmlspecialchars(strip_tags($this->decision_22));
      $this->decision_23 = htmlspecialchars(strip_tags($this->decision_23));
      $this->decision_24 = htmlspecialchars(strip_tags($this->decision_24));
      $this->decision_25 = htmlspecialchars(strip_tags($this->decision_25));
      $this->decision_26 = htmlspecialchars(strip_tags($this->decision_26));
      $this->decision_27 = htmlspecialchars(strip_tags($this->decision_27));
      $this->decision_28 = htmlspecialchars(strip_tags($this->decision_28));
      $this->decision_29 = htmlspecialchars(strip_tags($this->decision_29));
      $this->decision_30 = htmlspecialchars(strip_tags($this->decision_30));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);
      $stmt->bindParam(':name', $this->name);
      $stmt->bindParam(':pass', $this->pass);
      $stmt->bindParam(':no_of_decisions', $this->no_of_decisions);
      $stmt->bindParam(':decision_1', $this->decision_1);
      $stmt->bindParam(':decision_2', $this->decision_2);
      $stmt->bindParam(':decision_3', $this->decision_3);
      $stmt->bindParam(':decision_4', $this->decision_4);
      $stmt->bindParam(':decision_5', $this->decision_5);
      $stmt->bindParam(':decision_6', $this->decision_6);
      $stmt->bindParam(':decision_7', $this->decision_7);
      $stmt->bindParam(':decision_8', $this->decision_8);
      $stmt->bindParam(':decision_9', $this->decision_9);
      $stmt->bindParam(':decision_10', $this->decision_10);
      $stmt->bindParam(':decision_11', $this->decision_11);
      $stmt->bindParam(':decision_12', $this->decision_12);
      $stmt->bindParam(':decision_13', $this->decision_13);
      $stmt->bindParam(':decision_14', $this->decision_14);
      $stmt->bindParam(':decision_15', $this->decision_15);
      $stmt->bindParam(':decision_16', $this->decision_16);
      $stmt->bindParam(':decision_17', $this->decision_17);
      $stmt->bindParam(':decision_18', $this->decision_18);
      $stmt->bindParam(':decision_19', $this->decision_19);
      $stmt->bindParam(':decision_20', $this->decision_20);
      $stmt->bindParam(':decision_21', $this->decision_21);
      $stmt->bindParam(':decision_22', $this->decision_22);
      $stmt->bindParam(':decision_23', $this->decision_23);
      $stmt->bindParam(':decision_24', $this->decision_24);
      $stmt->bindParam(':decision_25', $this->decision_25);
      $stmt->bindParam(':decision_26', $this->decision_26);
      $stmt->bindParam(':decision_27', $this->decision_27);
      $stmt->bindParam(':decision_28', $this->decision_28);
      $stmt->bindParam(':decision_29', $this->decision_29);
      $stmt->bindParam(':decision_30', $this->decision_30);

      // Execute query
      if($stmt->execute()) {
        return $this->edit_existing_poll_settings();
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Edits the poll setting for weight and object
    public function edit_existing_poll_settings() {
      //everything on default (maybe it was not before...)
      if( $this->weight_settings == 0 && $this->object_settings == 0 )
        return $this->delete_poll_settings();

      //nothing to do, because array empty
      if( count($this->weight_settings) == 0 && count($this->object_settings) == 0 )
        return $this->delete_poll_settings();
        //or just return false?

      //first, delete all, then add only the not-default-one
      return $this->delete_poll_settings() && $this->create_poll_settings_weight() && $this->create_poll_settings_object();
    }


    // Delete settings by phash
    public function delete_poll_settings() {
      // Create query
      $query = 'DELETE FROM ' . $this->tableSettings . ' 
                WHERE phash = :phash';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Toggle Active
    public function toggle_active() {
      // Create query
      $query = 'UPDATE ' . $this->tablePoll . '
                SET active = :active
                WHERE phash = :phash';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      //toggle active
      if( $this->active == 1 )
        $this->active = 0;
      else
        $this->active = 1;

      // Clean data
      $this->active = htmlspecialchars(strip_tags($this->active));
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind data
      $stmt->bindParam(':active', $this->active);
      $stmt->bindParam(':phash', $this->phash);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Get all answer to a poll
    public function get_all_answers_by_poll() {
      // Create query
      $query = 'SELECT a.hash, cl.first_name, cl.last_name, cl.company, a.answer_1, a.answer_2, a.answer_3, 
                             a.answer_4, a.answer_5, a.answer_6, 
                             a.answer_7, a.answer_8, a.answer_9, a.answer_10, a.answer_11, a.answer_12, 
                             a.answer_13, a.answer_14, a.answer_15, a.answer_16, a.answer_17, 
                             a.answer_18, a.answer_19, a.answer_20, a.answer_21, a.answer_22, 
                             a.answer_23, a.answer_24, a.answer_25, a.answer_26, a.answer_27, 
                             a.answer_28, a.answer_29, a.answer_30
                FROM ' . $this->tableAnswer . ' a
                LEFT JOIN ' . $this->tableClient . ' cl ON a.hash = cl.hash
                WHERE phash = :phash';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);

      // Execute query
      if( $stmt->execute() ) {

        if( $stmt->rowCount() > 0 )
          return $stmt;

      }

      return null;
    }


    // Get all answers by decision number
    public function get_answers_by_decision_no( $decision ) {
      if( $decision === 0 || $decision === "" )
        return null;

      //if it is related to some objects (but not all)
      $obj_array = $this->get_poll_setting_object_by_phash_decision( $decision );
      $objString = "";
      if( is_array($obj_array) && count($obj_array) > 0 )
      {
        //first item
        $objString = "AND (";

        for( $i=0; $i<count($obj_array); $i++ ) {
          $objString .= "co.oid = " . $obj_array[$i];

          //not last, there is another one
          if( $i+1 < count($obj_array) )
            $objString .= " OR ";
        }

        //last item
        $objString .= ") ";
      }

      //debug purpose
      //error_log( "DEBUG: poll - get_answers_by_decision_no() - object settings... " . $objString, 0);

      $weightStr = $this->get_poll_setting_weight_by_phash_decision( $decision );

      $weightQuery = "";

      switch ($weightStr) {
          case "garage":
              $weightQuery = "AND co.garage_weight > 0 ";
              break;
          case "flat":
              $weightQuery = "AND co.flat_weight > 0 ";
              break;
          default: //"default"
              $weightQuery = "";
      }

      // Create query
      $query = 'SELECT DISTINCT a.hash, p.hid, cl.first_name, cl.last_name, cl.company, SUM(co.flat_weight) as flatWeight, SUM(co.garage_weight) as garageWeight, a.answer_X, p.active_decision as active_decision, COUNT(a.hash) as counts
                  FROM (
                          SELECT DISTINCT hash, answer_'.$decision.' as answer_X, phash
                            FROM ' . $this->tableAnswer . '
                            WHERE phash = :phash
                        ) a
                INNER JOIN ' . $this->tablePoll . ' p ON p.phash = a.phash
                INNER JOIN ' . $this->tableClient . ' cl ON cl.hash = a.hash
                INNER JOIN ' . $this->tableCorres . ' co ON co.pid = cl.pid
                WHERE co.hid = p.hid AND co.level = 1 ' . $objString . '' . $weightQuery . '
                GROUP BY a.hash';

      //error_log( "DEBUG: poll - get_answers_by_decision_no() - query: " . $query, 0);
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind data
      $stmt->bindParam(':phash', $this->phash);

      // Execute query
      if( $stmt->execute() ) {

        if( $stmt->rowCount() > 0 )
          return $stmt;
      }

      return null;
    }


    // Activate decision by phash
    public function activate_decision_by_poll( $decision ) {
      if( $decision === 0 )
        return false;

      // Create query
      $query = 'UPDATE ' . $this->tablePoll . '
                SET active_decision = :active_decision
                WHERE phash = :phash';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind data
      $stmt->bindParam(':active_decision', $decision );
      $stmt->bindParam(':phash', $this->phash);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Return hid of active poll
    public function get_hid_of_active_poll() {
      // Create query
      $query = 'SELECT p.hid
                FROM ' . $this->tablePoll . ' p
                WHERE p.active = 1';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      if( $stmt->execute() ) {

        $row = $stmt->fetch(PDO::FETCH_ASSOC);

        if( isset($row['hid']) )
          return $row['hid'];
      }

      return 0;
    }

}