<?php 
  class Proxy {
    
    // DB stuff
    private $conn;
    private $tableProxy = 'poll_proxy';
    private $tablePoll = 'poll';
    private $tableClient = 'client';
    private $tableCorres = 'correspondence';

    // Proxy Properties
    public $phash; 
    public $pid_absentee; 
    public $pid_agent;  

    // Constructor with DB
    public function __construct( $db ) {
      $this->conn = $db;
    }


    // Get proxy
    public function get_proxy() {
      // Create query
      /*$query = 'SELECT id, pid_absentee, pid_agent, phash
                FROM ' . $this->tableProxy . '
                WHERE phash = ?';*/

      $query = 'SELECT x.id as id, y.id as id2, x.pid_absentee as pid_absentee, x.first_name as abs_first_name, x.last_name as abs_last_name, x.company as abs_company, y.pid_agent as pid_agent, y.first_name as ag_first_name, y.last_name as ag_last_name, y.company as ag_company, x.phash as phash
                FROM (SELECT p.id, p.pid_absentee,  p.phash, c.first_name, c.last_name, c.company FROM ' . $this->tableProxy . ' p, ' . $this->tableClient . ' c WHERE phash = ? AND p.pid_absentee = c.pid) as x
                LEFT JOIN (SELECT p.id, p.pid_agent,  p.phash, c.first_name, c.last_name, c.company FROM ' . $this->tableProxy . ' p, ' . $this->tableClient . ' c WHERE phash = ? AND p.pid_agent = c.pid) as y
                ON x.id = y.id';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind ID
      $stmt->bindParam( 1, $this->phash );
      $stmt->bindParam( 2, $this->phash );

      // Execute query
      if( $stmt->execute() ) {

          return $stmt;
      }

      return null;
    }


    // Create proxy via absentee, agent and phash
    public function create_proxy() {
      // Create query
      $query = 'INSERT INTO ' . $this->tableProxy . '
                SET phash = :phash, pid_absentee = :pid_absentee, pid_agent = :pid_agent';
      
      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));
      $this->pid_absentee = htmlspecialchars(strip_tags($this->pid_absentee));
      $this->pid_agent = htmlspecialchars(strip_tags($this->pid_agent));

      // Bind ID
      $stmt->bindParam( ':phash', $this->phash );
      $stmt->bindParam( ':pid_absentee', $this->pid_absentee );
      $stmt->bindParam( ':pid_agent', $this->pid_agent );

      // Execute query
      if( $stmt->execute() ) {

          return 1;
      }

      return 0;
    }


    // Delete proxy items by id
    public function delete_proxy( $id ) {
      // Create query
      $query = 'DELETE FROM ' . $this->tableProxy . ' 
                WHERE id = ' . $id;

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Execute query
      if($stmt->execute()) {
        return true;
      }

      // Print error if something goes wrong
      //printf("Error: %s.\n", $stmt->error);

      return false;
    }


    // Get all proxy members for the dedicated house via phash
    public function get_proxy_of_house() {
      // Create query
      $query = 'SELECT cl.pid, cl.first_name, cl.last_name, /*cl.first_name2, cl.last_name2,*/ cl.company
                FROM ' . $this->tableClient . ' cl, ' . $this->tableCorres . ' co
                WHERE cl.pid = co.pid AND co.level = 1 AND co.hid = (SELECT hid FROM ' . $this->tablePoll . ' WHERE phash = ?)
                ORDER BY cl.last_name ASC';

      // Prepare statement
      $stmt = $this->conn->prepare($query);

      // Clean data
      $this->phash = htmlspecialchars(strip_tags($this->phash));

      // Bind ID
      $stmt->bindParam( 1, $this->phash );

      // Execute query
      if( $stmt->execute() ) {

          return $stmt;
      }

      return null;
    }

}